
// Name
//===================================
//
// Background
//


// DEPENDENCIES
//===================================

    #include "background.h"
    #include "camera.h"
    #include "game.h"


// STATIC PROPERTIES
//===================================


// METHODS
//===================================

    Background::Background
    (
        Graphics::SpriteSheet texture,
        int width,
        int height,
        int offset_x,
        int offset_y,
        double scroll_speed_x,
        double scroll_speed_y,
        int num_o_frames
    )
    :
        texture_ ( texture ),
        width_ ( width ),
        height_ ( height ),
        offset_x_ ( offset_x ),
        offset_y_ ( offset_y ),
        x_ ( offset_x ),
        y_ ( offset_y ),
        scroll_speed_x_ ( scroll_speed_x ),
        scroll_speed_y_ ( scroll_speed_y ),
        num_o_frames_ ( num_o_frames ),
        current_frame_ ( Counter( 0, num_o_frames, 0, true ) )
    {};

    Background::~Background() {};

    void Background::render( Graphics& graphics, Camera& camera )
    {
        if ( texture_ >= 0 && texture_ < Graphics::SpriteSheet::LIMIT )
        {
            renderY( graphics, camera );
        }

        if ( Game::nextFrame() )
        {
            ++current_frame_;
        }
    };

    void Background::renderY( Graphics& graphics, Camera& camera )
    {
        int st = offset_y_ + ( (int)( camera.y() * scroll_speed_y_ ) % height_ );
        int dt = 0;
        int bottom = std::min( height_ - st, camera.heightPixels() );
        int far_bottom = 0;

        renderX( graphics, camera, dt, st, bottom );

        while ( far_bottom < camera.heightPixels() )
        {
            far_bottom += bottom;
            dt += bottom;
            bottom = std::min( height_, camera.heightPixels() - dt );

            renderX( graphics, camera, dt, 0, bottom );
        }
    };

    void Background::renderX( Graphics& graphics, Camera& camera, int dy, int sy, int h )
    {
        int sl = offset_x_ + ( (int)( camera.x() * scroll_speed_x_ ) % width_ ) + ( width_ * current_frame_.value() );
        int dl = 0;
        int right = std::min( width_ - ( sl - ( width_ * current_frame_.value() ) ), camera.widthPixels() );
        int far_right = 0;

        SDL_Rect dest = { dl, dy, right, h };
        SDL_Rect source = { sl, sy, right, h };

        graphics.renderObject( texture_, source, dest );

        while ( far_right < camera.widthPixels() )
        {
            far_right += right;
            dl += right;
            right = std::min( width_, camera.widthPixels() - dl );

            dest = { dl, dy, right, h };
            source = { ( width_ * current_frame_.value() ), sy, right, h };

            graphics.renderObject( texture_, source, dest );
        }
    };
