
// Name
//===================================
//
// BlockSystem
//


// DEPENDENCIES
//===================================

    #include "block_component_climbable.h"
    #include "block_component_dropping.h"
    #include "block_component_goal.h"
    #include "block_component_high_slope_left.h"
    #include "block_component_high_slope_right.h"
    #include "block_component_heal.h"
    #include "block_component_hurt.h"
    #include "block_component_low_slope_left.h"
    #include "block_component_low_slope_right.h"
    #include "block_component_message.h"
    #include "block_component_money.h"
    #include "block_component_single_use.h"
    #include "block_component_solid.h"
    #include "block_component_warp.h"
    #include "block_condition_collide_any.h"
    #include "block_condition_collide_bottom.h"
    #include "block_condition_collide_top.h"
    #include "block_condition_hero.h"
    #include "block_system.h"
    #include "block_type.h"
    #include "camera.h"
    #include "graphics.h"
    #include <iostream>
    #include "map.h"
    #include "sprite.h"
    #include "sprite_graphics.h"


// STATIC VARIABLES
//===================================


// METHODS
//===================================

    BlockSystem::BlockSystem()
    {
        block_types_ =
        {
            new BlockType
            (
                new SpriteGraphics( Graphics::SpriteSheet::LVTILESET_CITY, 12*8, 0 ),
                new SpriteGraphics( Graphics::SpriteSheet::LVTILESET_CITY, 13*8, 0 ),
                new SpriteGraphics( Graphics::SpriteSheet::LVTILESET_CITY, 12*8, 8 ),
                new SpriteGraphics( Graphics::SpriteSheet::LVTILESET_CITY, 13*8, 8 ),
                { new BlockComponentMoney( 100 ), new BlockComponentSingleUse() }, { { new BlockConditionCollideAny(), new BlockConditionHero() }, { new BlockConditionCollideAny(), new BlockConditionHero() } }
            ),
            new BlockType
            (
                new SpriteGraphics( Graphics::SpriteSheet::LVTILESET_CITY, 120, 32 ),
                new SpriteGraphics( Graphics::SpriteSheet::LVTILESET_CITY, 128, 32 ),
                new SpriteGraphics( Graphics::SpriteSheet::LVTILESET_CITY, 136, 32 ),
                new SpriteGraphics( Graphics::SpriteSheet::LVTILESET_CITY, 144, 32 ),
                { new BlockComponentMoney( 250 ), new BlockComponentSingleUse() }, { { new BlockConditionCollideAny(), new BlockConditionHero() }, { new BlockConditionCollideAny(), new BlockConditionHero() } }
            ),
            new BlockType
            (
                new SpriteGraphics( Graphics::SpriteSheet::LVTILESET_CITY, 152, 32 ),
                new SpriteGraphics( Graphics::SpriteSheet::LVTILESET_CITY, 160, 32 ),
                new SpriteGraphics( Graphics::SpriteSheet::LVTILESET_CITY, 168, 32 ),
                new SpriteGraphics( Graphics::SpriteSheet::LVTILESET_CITY, 176, 32 ),
                { new BlockComponentMoney( 500 ), new BlockComponentSingleUse() }, { { new BlockConditionCollideAny(), new BlockConditionHero() }, { new BlockConditionCollideAny(), new BlockConditionHero() } }
            ),
            new BlockType
            (
                new SpriteGraphics( Graphics::SpriteSheet::LVTILESET_CITY, 184, 32 ),
                new SpriteGraphics( Graphics::SpriteSheet::LVTILESET_CITY, 192, 32 ),
                new SpriteGraphics( Graphics::SpriteSheet::LVTILESET_CITY, 200, 32 ),
                new SpriteGraphics( Graphics::SpriteSheet::LVTILESET_CITY, 208, 32 ),
                { new BlockComponentMessage(), new BlockComponentSolid() }, { { new BlockConditionCollideTop(), new BlockConditionHero() } }
            ),
            new BlockType
            (
                new SpriteGraphics( Graphics::SpriteSheet::LVTILESET_CITY, 216, 32 ),
                new SpriteGraphics( Graphics::SpriteSheet::LVTILESET_CITY, 224, 32 ),
                new SpriteGraphics( Graphics::SpriteSheet::LVTILESET_CITY, 232, 32 ),
                new SpriteGraphics( Graphics::SpriteSheet::LVTILESET_CITY, 240, 32 ),
                { new BlockComponentHeal(), new BlockComponentSingleUse() }, { { new BlockConditionCollideAny(), new BlockConditionHero() }, { new BlockConditionCollideAny(), new BlockConditionHero() } }
            ),
            nullptr,
            nullptr,
            nullptr,
            nullptr,
            nullptr,
            nullptr,
            nullptr,
            nullptr,
            nullptr,
            nullptr,
            nullptr,
        };

        block_types_.push_back( new BlockType
        (
            nullptr,
            nullptr,
            new SpriteGraphics( Graphics::SpriteSheet::LVTILESET_CITY, 128, 24, false, false, 0, true ),
            new SpriteGraphics( Graphics::SpriteSheet::LVTILESET_CITY, 128, 24, false, false, 0, true ),
            {}//{}//BlockType::BlockAction::__NULL
        ));
        block_types_.push_back( new BlockType
        (
            nullptr,
            nullptr,
            new SpriteGraphics( Graphics::SpriteSheet::LVTILESET_CITY, 128, 24, false, false, 0, true ),
            new SpriteGraphics( Graphics::SpriteSheet::LVTILESET_CITY, 168, 24, false, false, 0, true ),
            {}//{}//BlockType::BlockAction::__NULL
        ));
        block_types_.push_back( new BlockType
        (
            nullptr,
            nullptr,
            new SpriteGraphics( Graphics::SpriteSheet::LVTILESET_CITY, 168, 24, true, false, 0, true ),
            new SpriteGraphics( Graphics::SpriteSheet::LVTILESET_CITY, 128, 24, false, false, 0, true ),
            {}//{}//BlockType::BlockAction::__NULL
        ));

        block_types_.push_back( new BlockType
        (
            new SpriteGraphics( Graphics::SpriteSheet::LVTILESET_CITY, 200, 24 ),
            new SpriteGraphics( Graphics::SpriteSheet::LVTILESET_CITY, 208, 24 ),
            new SpriteGraphics( Graphics::SpriteSheet::LVTILESET_CITY, 200, 16 ),
            new SpriteGraphics( Graphics::SpriteSheet::LVTILESET_CITY, 208, 16 ),
            { new BlockComponentSolid() }
        ));
        block_types_.push_back( new BlockType
        (
            new SpriteGraphics( Graphics::SpriteSheet::LVTILESET_CITY, 216, 24 ),
            new SpriteGraphics( Graphics::SpriteSheet::LVTILESET_CITY, 232, 24 ),
            new SpriteGraphics( Graphics::SpriteSheet::LVTILESET_CITY, 248, 24 ),
            new SpriteGraphics( Graphics::SpriteSheet::LVTILESET_CITY, 0, 32 ),
            { new BlockComponentSolid() }
        ));
        block_types_.push_back( new BlockType
        (
            new SpriteGraphics( Graphics::SpriteSheet::LVTILESET_CITY, 232, 24 ),
            new SpriteGraphics( Graphics::SpriteSheet::LVTILESET_CITY, 240, 24 ),
            new SpriteGraphics( Graphics::SpriteSheet::LVTILESET_CITY, 8, 32 ),
            new SpriteGraphics( Graphics::SpriteSheet::LVTILESET_CITY, 16, 32 ),
            { new BlockComponentSolid() }
        ));

        block_types_.push_back( nullptr );
        block_types_.push_back( nullptr );
        block_types_.push_back( nullptr );
        block_types_.push_back( nullptr );
        block_types_.push_back( nullptr );
        block_types_.push_back( nullptr );
        block_types_.push_back( nullptr );
        block_types_.push_back( nullptr );
        block_types_.push_back( nullptr );
        block_types_.push_back( nullptr );

        block_types_.push_back( new BlockType
        (
            new SpriteGraphics( Graphics::SpriteSheet::LVTILESET_CITY, 112, 24 ),
            new SpriteGraphics( Graphics::SpriteSheet::LVTILESET_CITY, 120, 24 ),
            new SpriteGraphics( Graphics::SpriteSheet::LVTILESET_CITY, 136, 24 ),
            new SpriteGraphics( Graphics::SpriteSheet::LVTILESET_CITY, 136, 24 ),
            { new BlockComponentSolid() }
        ));
        block_types_.push_back( new BlockType
        (
            new SpriteGraphics( Graphics::SpriteSheet::LVTILESET_CITY, 112, 24 ),
            new SpriteGraphics( Graphics::SpriteSheet::LVTILESET_CITY, 160, 24 ),
            new SpriteGraphics( Graphics::SpriteSheet::LVTILESET_CITY, 136, 24 ),
            new SpriteGraphics( Graphics::SpriteSheet::LVTILESET_CITY, 152, 24 ),
            { new BlockComponentSolid() }
        ));
        block_types_.push_back( new BlockType
        (
            new SpriteGraphics( Graphics::SpriteSheet::LVTILESET_CITY, 160, 24, true ),
            new SpriteGraphics( Graphics::SpriteSheet::LVTILESET_CITY, 120, 24 ),
            new SpriteGraphics( Graphics::SpriteSheet::LVTILESET_CITY, 144, 24 ),
            new SpriteGraphics( Graphics::SpriteSheet::LVTILESET_CITY, 136, 24 ),
            { new BlockComponentSolid() }
        ));

        block_types_.push_back( new BlockType
        (
            new SpriteGraphics( Graphics::SpriteSheet::LVTILESET_CITY, 24, 32 ),
            new SpriteGraphics( Graphics::SpriteSheet::LVTILESET_CITY, 32, 32 ),
            new SpriteGraphics( Graphics::SpriteSheet::LVTILESET_CITY, 72, 32 ),
            new SpriteGraphics( Graphics::SpriteSheet::LVTILESET_CITY, 80, 32 ),
            { new BlockComponentSolid() }
        ));
        block_types_.push_back( new BlockType
        (
            new SpriteGraphics( Graphics::SpriteSheet::LVTILESET_CITY, 40, 32 ),
            new SpriteGraphics( Graphics::SpriteSheet::LVTILESET_CITY, 48, 32 ),
            new SpriteGraphics( Graphics::SpriteSheet::LVTILESET_CITY, 88, 32 ),
            new SpriteGraphics( Graphics::SpriteSheet::LVTILESET_CITY, 96, 32 ),
            { new BlockComponentSolid() }
        ));
        block_types_.push_back( new BlockType
        (
            new SpriteGraphics( Graphics::SpriteSheet::LVTILESET_CITY, 56, 32 ),
            new SpriteGraphics( Graphics::SpriteSheet::LVTILESET_CITY, 64, 32 ),
            new SpriteGraphics( Graphics::SpriteSheet::LVTILESET_CITY, 104, 32 ),
            new SpriteGraphics( Graphics::SpriteSheet::LVTILESET_CITY, 112, 32 ),
            { new BlockComponentSolid() }
        ));

        block_types_.push_back( new BlockType
        (
            new SpriteGraphics( Graphics::SpriteSheet::LVTILESET_CITY, 80, 40 ),
            new SpriteGraphics( Graphics::SpriteSheet::LVTILESET_CITY, 88, 40 ),
            new SpriteGraphics( Graphics::SpriteSheet::LVTILESET_CITY, 96, 40 ),
            new SpriteGraphics( Graphics::SpriteSheet::LVTILESET_CITY, 104, 40 ),
            { new BlockComponentSolid(), new BlockComponentDropping() }, { { new BlockConditionCollideAny() }, { new BlockConditionCollideAny(), new BlockConditionHero() } }
        ));

        block_types_.push_back( new BlockType
        (
            nullptr,
            nullptr,
            new SpriteGraphics( Graphics::SpriteSheet::LVTILESET_CITY, 32, 40 ),
            new SpriteGraphics( Graphics::SpriteSheet::LVTILESET_CITY, 40, 40 ),
            { new BlockComponentLowSlopeLeft() }
        ));

        block_types_.push_back( new BlockType
        (
            new SpriteGraphics( Graphics::SpriteSheet::LVTILESET_CITY, 32, 40 ),
            new SpriteGraphics( Graphics::SpriteSheet::LVTILESET_CITY, 40, 40 ),
            new SpriteGraphics( Graphics::SpriteSheet::LVTILESET_CITY, 16, 40 ),
            new SpriteGraphics( Graphics::SpriteSheet::LVTILESET_CITY, 24, 40 ),
            { new BlockComponentHighSlopeLeft() }
        ));
        block_types_.push_back( new BlockType
        (
            new SpriteGraphics( Graphics::SpriteSheet::LVTILESET_CITY, 40, 40, true ),
            new SpriteGraphics( Graphics::SpriteSheet::LVTILESET_CITY, 32, 40, true ),
            new SpriteGraphics( Graphics::SpriteSheet::LVTILESET_CITY, 24, 40, true ),
            new SpriteGraphics( Graphics::SpriteSheet::LVTILESET_CITY, 16, 40, true ),
            { new BlockComponentHighSlopeRight() }
        ));

        block_types_.push_back( new BlockType
        (
            nullptr,
            nullptr,
            new SpriteGraphics( Graphics::SpriteSheet::LVTILESET_CITY, 40, 40, true ),
            new SpriteGraphics( Graphics::SpriteSheet::LVTILESET_CITY, 32, 40, true ),
            { new BlockComponentLowSlopeRight() }
        ));

        block_types_.push_back( nullptr );
        block_types_.push_back( nullptr );
        block_types_.push_back( nullptr );
        block_types_.push_back( nullptr );
        block_types_.push_back( nullptr );


        block_types_.push_back( new BlockType
        (
            new SpriteGraphics( Graphics::SpriteSheet::LVTILESET_CITY, 136, 24 ),
            new SpriteGraphics( Graphics::SpriteSheet::LVTILESET_CITY, 136, 24 ),
            new SpriteGraphics( Graphics::SpriteSheet::LVTILESET_CITY, 136, 24 ),
            new SpriteGraphics( Graphics::SpriteSheet::LVTILESET_CITY, 136, 24 ),
            { new BlockComponentSolid() }
        ));
        block_types_.push_back( new BlockType
        (
            new SpriteGraphics( Graphics::SpriteSheet::LVTILESET_CITY, 136, 24 ),
            new SpriteGraphics( Graphics::SpriteSheet::LVTILESET_CITY, 152, 24 ),
            new SpriteGraphics( Graphics::SpriteSheet::LVTILESET_CITY, 136, 24 ),
            new SpriteGraphics( Graphics::SpriteSheet::LVTILESET_CITY, 152, 24 ),
            { new BlockComponentSolid() }
        ));
        block_types_.push_back( new BlockType
        (
            new SpriteGraphics( Graphics::SpriteSheet::LVTILESET_CITY, 144, 24 ),
            new SpriteGraphics( Graphics::SpriteSheet::LVTILESET_CITY, 136, 24 ),
            new SpriteGraphics( Graphics::SpriteSheet::LVTILESET_CITY, 144, 24 ),
            new SpriteGraphics( Graphics::SpriteSheet::LVTILESET_CITY, 136, 24 ),
            { new BlockComponentSolid() }
        ));

        block_types_.push_back( new BlockType
        (
            new SpriteGraphics( Graphics::SpriteSheet::LVTILESET_CITY, 184, 16 ),
            new SpriteGraphics( Graphics::SpriteSheet::LVTILESET_CITY, 192, 16 ),
            new SpriteGraphics( Graphics::SpriteSheet::LVTILESET_CITY, 184, 24, false, false, 0, true ),
            new SpriteGraphics( Graphics::SpriteSheet::LVTILESET_CITY, 192, 24, false, false, 0, true ),
            { new BlockComponentSolid() }
        ));
        block_types_.push_back( new BlockType
        (
            new SpriteGraphics( Graphics::SpriteSheet::LVTILESET_CITY, 200, 24 ),
            new SpriteGraphics( Graphics::SpriteSheet::LVTILESET_CITY, 208, 24 ),
            new SpriteGraphics( Graphics::SpriteSheet::LVTILESET_CITY, 72, 32, false, false, 0, true ),
            new SpriteGraphics( Graphics::SpriteSheet::LVTILESET_CITY, 80, 32, false, false, 0, true ),
            { new BlockComponentSolid() }
        ));
        block_types_.push_back( new BlockType
        (
            new SpriteGraphics( Graphics::SpriteSheet::LVTILESET_CITY, 232, 24 ),
            new SpriteGraphics( Graphics::SpriteSheet::LVTILESET_CITY, 240, 24 ),
            new SpriteGraphics( Graphics::SpriteSheet::LVTILESET_CITY, 104, 32, false, false, 0, true ),
            new SpriteGraphics( Graphics::SpriteSheet::LVTILESET_CITY, 112, 32, false, false, 0, true ),
            { new BlockComponentSolid() }
        ));

        block_types_.push_back( new BlockType
        (
            new SpriteGraphics( Graphics::SpriteSheet::LVTILESET_CITY, 0, 40 ),
            new SpriteGraphics( Graphics::SpriteSheet::LVTILESET_CITY, 0, 40 ),
            new SpriteGraphics( Graphics::SpriteSheet::LVTILESET_CITY, 48, 40 ),
            new SpriteGraphics( Graphics::SpriteSheet::LVTILESET_CITY, 48, 40 ),
            { new BlockComponentSolid() }
        ));
        block_types_.push_back( new BlockType
        (
            new SpriteGraphics( Graphics::SpriteSheet::LVTILESET_CITY, 16, 40 ),
            new SpriteGraphics( Graphics::SpriteSheet::LVTILESET_CITY, 16, 40 ),
            new SpriteGraphics( Graphics::SpriteSheet::LVTILESET_CITY, 64, 40 ),
            new SpriteGraphics( Graphics::SpriteSheet::LVTILESET_CITY, 72, 40 ),
            { new BlockComponentSolid() }
        ));
        block_types_.push_back( new BlockType
        (
            new SpriteGraphics( Graphics::SpriteSheet::LVTILESET_CITY, 64, 40 ),
            new SpriteGraphics( Graphics::SpriteSheet::LVTILESET_CITY, 72, 40 ),
            new SpriteGraphics( Graphics::SpriteSheet::LVTILESET_CITY, 72, 40 ),
            new SpriteGraphics( Graphics::SpriteSheet::LVTILESET_CITY, 72, 40 ),
            { new BlockComponentSolid() }
        ));
        block_types_.push_back( new BlockType
        (
            new SpriteGraphics( Graphics::SpriteSheet::LVTILESET_CITY, 72, 40, true ),
            new SpriteGraphics( Graphics::SpriteSheet::LVTILESET_CITY, 64, 40, true ),
            new SpriteGraphics( Graphics::SpriteSheet::LVTILESET_CITY, 72, 40, true ),
            new SpriteGraphics( Graphics::SpriteSheet::LVTILESET_CITY, 72, 40, true ),
            { new BlockComponentSolid() }
        ));
        block_types_.push_back( new BlockType
        (
            new SpriteGraphics( Graphics::SpriteSheet::LVTILESET_CITY, 16, 40, true ),
            new SpriteGraphics( Graphics::SpriteSheet::LVTILESET_CITY, 16, 40, true ),
            new SpriteGraphics( Graphics::SpriteSheet::LVTILESET_CITY, 72, 40, true ),
            new SpriteGraphics( Graphics::SpriteSheet::LVTILESET_CITY, 64, 40, true ),
            { new BlockComponentSolid() }
        ));
        block_types_.push_back( new BlockType
        (
            new SpriteGraphics( Graphics::SpriteSheet::LVTILESET_CITY, 72, 40 ),
            new SpriteGraphics( Graphics::SpriteSheet::LVTILESET_CITY, 72, 40 ),
            new SpriteGraphics( Graphics::SpriteSheet::LVTILESET_CITY, 72, 40 ),
            new SpriteGraphics( Graphics::SpriteSheet::LVTILESET_CITY, 72, 40 ),
            { new BlockComponentSolid() }
        ));

        block_types_.push_back( nullptr );
        block_types_.push_back( nullptr );
        block_types_.push_back( nullptr );
        block_types_.push_back( nullptr );

        block_types_.push_back( new BlockType
        (
            new SpriteGraphics( Graphics::SpriteSheet::LVTILESET_CITY, 0, 0 ),
            new SpriteGraphics( Graphics::SpriteSheet::LVTILESET_CITY, 8, 0 ),
            new SpriteGraphics( Graphics::SpriteSheet::LVTILESET_CITY, 16, 0 ),
            new SpriteGraphics( Graphics::SpriteSheet::LVTILESET_CITY, 16, 0 ),
            { new BlockComponentSolid() }
        ));
        block_types_.push_back( new BlockType
        (
            new SpriteGraphics( Graphics::SpriteSheet::LVTILESET_CITY, 16, 8 ),
            new SpriteGraphics( Graphics::SpriteSheet::LVTILESET_CITY, 16, 8 ),
            new SpriteGraphics( Graphics::SpriteSheet::LVTILESET_CITY, 16, 8 ),
            new SpriteGraphics( Graphics::SpriteSheet::LVTILESET_CITY, 16, 8 ),
            { new BlockComponentSolid() }
        ));
        block_types_.push_back( new BlockType
        (
            new SpriteGraphics( Graphics::SpriteSheet::LVTILESET_CITY, 24, 0 ),
            new SpriteGraphics( Graphics::SpriteSheet::LVTILESET_CITY, 24, 0 ),
            new SpriteGraphics( Graphics::SpriteSheet::LVTILESET_CITY, 24, 0 ),
            new SpriteGraphics( Graphics::SpriteSheet::LVTILESET_CITY, 24, 0 ),
            {}//{}//BlockType::BlockAction::__NULL
        ));
        block_types_.push_back( new BlockType
        (
            new SpriteGraphics( Graphics::SpriteSheet::LVTILESET_CITY, 32, 0 ),
            new SpriteGraphics( Graphics::SpriteSheet::LVTILESET_CITY, 40, 0 ),
            new SpriteGraphics( Graphics::SpriteSheet::LVTILESET_CITY, 32, 8 ),
            new SpriteGraphics( Graphics::SpriteSheet::LVTILESET_CITY, 40, 8 ),
            { new BlockComponentSolid() }
        ));
        block_types_.push_back( new BlockType
        (
            new SpriteGraphics( Graphics::SpriteSheet::LVTILESET_CITY, 64, 8 ),
            new SpriteGraphics( Graphics::SpriteSheet::LVTILESET_CITY, 72, 8 ),
            new SpriteGraphics( Graphics::SpriteSheet::LVTILESET_CITY, 64, 8 ),
            new SpriteGraphics( Graphics::SpriteSheet::LVTILESET_CITY, 72, 8 ),
            { new BlockComponentClimbable() }, { { new BlockConditionCollideAny() } }
        ));
        block_types_.push_back( new BlockType
        (
            new SpriteGraphics( Graphics::SpriteSheet::LVTILESET_CITY, 80, 0 ),
            new SpriteGraphics( Graphics::SpriteSheet::LVTILESET_CITY, 88, 0 ),
            new SpriteGraphics( Graphics::SpriteSheet::LVTILESET_CITY, 80, 8 ),
            new SpriteGraphics( Graphics::SpriteSheet::LVTILESET_CITY, 88, 8 ),
            {}//{}//BlockType::BlockAction::__NULL
        ));
        block_types_.push_back( new BlockType
        (
            new SpriteGraphics( Graphics::SpriteSheet::LVTILESET_CITY, 14*8, 0 ),
            new SpriteGraphics( Graphics::SpriteSheet::LVTILESET_CITY, 15*8, 0 ),
            new SpriteGraphics( Graphics::SpriteSheet::LVTILESET_CITY, 16*8, 0 ),
            new SpriteGraphics( Graphics::SpriteSheet::LVTILESET_CITY, 17*8, 0 ),
            {}//{}//BlockType::BlockAction::__NULL
        ));
        block_types_.push_back( new BlockType
        (
            new SpriteGraphics( Graphics::SpriteSheet::LVTILESET_CITY, 22*8, 0 ),
            new SpriteGraphics( Graphics::SpriteSheet::LVTILESET_CITY, 23*8, 0 ),
            new SpriteGraphics( Graphics::SpriteSheet::LVTILESET_CITY, 23*8, 0 ),
            new SpriteGraphics( Graphics::SpriteSheet::LVTILESET_CITY, 22*8, 0 ),
            { new BlockComponentSolid() }
        ));
        block_types_.push_back( new BlockType
        (
            new SpriteGraphics( Graphics::SpriteSheet::LVTILESET_CITY, 23*8, 8, false, false, 0, true ),
            new SpriteGraphics( Graphics::SpriteSheet::LVTILESET_CITY, 24*8, 8, false, false, 0, true ),
            new SpriteGraphics( Graphics::SpriteSheet::LVTILESET_CITY, 16, 0, false, false, 0, true ),
            new SpriteGraphics( Graphics::SpriteSheet::LVTILESET_CITY, 16, 0, false, false, 0, true ),
            {}//{}//BlockType::BlockAction::__NULL
        ));
        block_types_.push_back( new BlockType
        (
            new SpriteGraphics( Graphics::SpriteSheet::LVTILESET_CITY, 25*8, 8, false, false, 0, true ),
            new SpriteGraphics( Graphics::SpriteSheet::LVTILESET_CITY, 26*8, 8, false, false, 0, true ),
            new SpriteGraphics( Graphics::SpriteSheet::LVTILESET_CITY, 16, 0, false, false, 0, true ),
            new SpriteGraphics( Graphics::SpriteSheet::LVTILESET_CITY, 16, 0, false, false, 0, true ),
            { new BlockComponentClimbable() }, { { new BlockConditionCollideAny(), new BlockConditionHero() } }
        ));
        block_types_.push_back( new BlockType
        (
            new SpriteGraphics( Graphics::SpriteSheet::LVTILESET_CITY, 22*8, 0 ),
            new SpriteGraphics( Graphics::SpriteSheet::LVTILESET_CITY, 23*8, 0 ),
            new SpriteGraphics( Graphics::SpriteSheet::LVTILESET_CITY, 24*8, 0 ),
            new SpriteGraphics( Graphics::SpriteSheet::LVTILESET_CITY, 25*8, 0 ),
            { new BlockComponentSolid() }
        ));
        block_types_.push_back( new BlockType
        (
            new SpriteGraphics( Graphics::SpriteSheet::LVTILESET_CITY, 29*8, 8 ),
            new SpriteGraphics( Graphics::SpriteSheet::LVTILESET_CITY, 30*8, 8 ),
            new SpriteGraphics( Graphics::SpriteSheet::LVTILESET_CITY, 29*8, 16 ),
            new SpriteGraphics( Graphics::SpriteSheet::LVTILESET_CITY, 30*8, 16 ),
            { new BlockComponentHurt(), new BlockComponentSolid() }, { { new BlockConditionHero(), new BlockConditionCollideBottom() } }
        ));
        block_types_.push_back( new BlockType
        (
            new SpriteGraphics( Graphics::SpriteSheet::LVTILESET_CITY, 31*8, 0 ),
            new SpriteGraphics( Graphics::SpriteSheet::LVTILESET_CITY, 31*8, 0, true ),
            new SpriteGraphics( Graphics::SpriteSheet::LVTILESET_CITY, 31*8, 8 ),
            new SpriteGraphics( Graphics::SpriteSheet::LVTILESET_CITY, 31*8, 8, true ),
            {}//{}//BlockType::BlockAction::__NULL
        ));
        block_types_.push_back( new BlockType
        (
            new SpriteGraphics( Graphics::SpriteSheet::LVTILESET_CITY, 80, 16 ),
            new SpriteGraphics( Graphics::SpriteSheet::LVTILESET_CITY, 88, 16 ),
            new SpriteGraphics( Graphics::SpriteSheet::LVTILESET_CITY, 80, 24 ),
            new SpriteGraphics( Graphics::SpriteSheet::LVTILESET_CITY, 88, 24 ),
            { new BlockComponentSolid() }
        ));
        block_types_.push_back( new BlockType
        (
            new SpriteGraphics( Graphics::SpriteSheet::LVTILESET_CITY, 7*8, 24, 0, 0, 90 ),
            new SpriteGraphics( Graphics::SpriteSheet::LVTILESET_CITY, 7*8, 16, 0, 0, 90 ),
            new SpriteGraphics( Graphics::SpriteSheet::LVTILESET_CITY, 64, 24, 0, 0, 90 ),
            new SpriteGraphics( Graphics::SpriteSheet::LVTILESET_CITY, 64, 16, 0, 0, 90 ),
            { new BlockComponentSolid() }
        ));
        block_types_.push_back( new BlockType
        (
            new SpriteGraphics( Graphics::SpriteSheet::LVTILESET_CITY, 3*8, 8 ),
            new SpriteGraphics( Graphics::SpriteSheet::LVTILESET_CITY, 3*8, 8 ),
            new SpriteGraphics( Graphics::SpriteSheet::LVTILESET_CITY, 16, 16 ),
            new SpriteGraphics( Graphics::SpriteSheet::LVTILESET_CITY, 16, 16 ),
            { new BlockComponentSolid() }
        ));
        block_types_.push_back( new BlockType
        (
            new SpriteGraphics( Graphics::SpriteSheet::LVTILESET_CITY, 9*8, 0 ),
            new SpriteGraphics( Graphics::SpriteSheet::LVTILESET_CITY, 9*8, 0 ),
            new SpriteGraphics( Graphics::SpriteSheet::LVTILESET_CITY, 24, 0 ),
            new SpriteGraphics( Graphics::SpriteSheet::LVTILESET_CITY, 24, 0 ),
            { new BlockComponentSolid() }, { { new BlockConditionCollideBottom() } }
        ));
        block_types_.push_back( new BlockType
        (
            new SpriteGraphics( Graphics::SpriteSheet::LVTILESET_CITY, 24, 0 ),
            new SpriteGraphics( Graphics::SpriteSheet::LVTILESET_CITY, 24, 0 ),
            new SpriteGraphics( Graphics::SpriteSheet::LVTILESET_CITY, 64, 0 ),
            new SpriteGraphics( Graphics::SpriteSheet::LVTILESET_CITY, 64, 0 ),
            {}//{}//BlockType::BlockAction::__NULL
        ));
        block_types_.push_back( new BlockType
        (
            new SpriteGraphics( Graphics::SpriteSheet::LVTILESET_CITY, 18*8, 8 ),
            new SpriteGraphics( Graphics::SpriteSheet::LVTILESET_CITY, 18*8, 8 ),
            new SpriteGraphics( Graphics::SpriteSheet::LVTILESET_CITY, 18*8, 8 ),
            new SpriteGraphics( Graphics::SpriteSheet::LVTILESET_CITY, 18*8, 8 ),
            {}//{}//BlockType::BlockAction::__NULL
        ));
        block_types_.push_back( new BlockType
        (
            new SpriteGraphics( Graphics::SpriteSheet::LVTILESET_CITY, 19*8, 8 ),
            new SpriteGraphics( Graphics::SpriteSheet::LVTILESET_CITY, 19*8, 8 ),
            new SpriteGraphics( Graphics::SpriteSheet::LVTILESET_CITY, 18*8, 8 ),
            new SpriteGraphics( Graphics::SpriteSheet::LVTILESET_CITY, 18*8, 8 ),
            { new BlockComponentSolid() }, { { new BlockConditionCollideBottom() } }
        ));
        block_types_.push_back( new BlockType
        (
            new SpriteGraphics( Graphics::SpriteSheet::LVTILESET_CITY, 18*8, 8 ),
            new SpriteGraphics( Graphics::SpriteSheet::LVTILESET_CITY, 18*8, 8 ),
            new SpriteGraphics( Graphics::SpriteSheet::LVTILESET_CITY, 20*8, 8 ),
            new SpriteGraphics( Graphics::SpriteSheet::LVTILESET_CITY, 20*8, 8 ),
            {}//{}//BlockType::BlockAction::__NULL
        ));
        block_types_.push_back( new BlockType
        (
            new SpriteGraphics( Graphics::SpriteSheet::LVTILESET_CITY, 21*8, 8 ),
            new SpriteGraphics( Graphics::SpriteSheet::LVTILESET_CITY, 22*8, 8 ),
            new SpriteGraphics( Graphics::SpriteSheet::LVTILESET_CITY, 21*8, 16 ),
            new SpriteGraphics( Graphics::SpriteSheet::LVTILESET_CITY, 22*8, 16 ),
            {}//{}//BlockType::BlockAction::__NULL
        ));
        block_types_.push_back( new BlockType
        (
            new SpriteGraphics( Graphics::SpriteSheet::LVTILESET_CITY, 18*8, 0 ),
            new SpriteGraphics( Graphics::SpriteSheet::LVTILESET_CITY, 19*8, 0 ),
            new SpriteGraphics( Graphics::SpriteSheet::LVTILESET_CITY, 20*8, 0 ),
            new SpriteGraphics( Graphics::SpriteSheet::LVTILESET_CITY, 21*8, 0 ),
            {}//{}//BlockType::BlockAction::__NULL
        ));
        block_types_.push_back( new BlockType
        (
            new SpriteGraphics( Graphics::SpriteSheet::LVTILESET_CITY, 6*8, 0 ),
            new SpriteGraphics( Graphics::SpriteSheet::LVTILESET_CITY, 7*8, 0 ),
            new SpriteGraphics( Graphics::SpriteSheet::LVTILESET_CITY, 22*8, 0 ),
            new SpriteGraphics( Graphics::SpriteSheet::LVTILESET_CITY, 23*8, 0 ),
            { new BlockComponentSolid() }
        ));
        block_types_.push_back( new BlockType
        (
            new SpriteGraphics( Graphics::SpriteSheet::LVTILESET_CITY, 27*8, 0 ),
            new SpriteGraphics( Graphics::SpriteSheet::LVTILESET_CITY, 28*8, 0 ),
            new SpriteGraphics( Graphics::SpriteSheet::LVTILESET_CITY, 2*8, 0 ),
            new SpriteGraphics( Graphics::SpriteSheet::LVTILESET_CITY, 2*8, 0 ),
            { new BlockComponentSolid() }
        ));
        block_types_.push_back( new BlockType
        (
            new SpriteGraphics( Graphics::SpriteSheet::LVTILESET_CITY, 29*8, 0 ),
            new SpriteGraphics( Graphics::SpriteSheet::LVTILESET_CITY, 30*8, 0 ),
            new SpriteGraphics( Graphics::SpriteSheet::LVTILESET_CITY, 2*8, 0 ),
            new SpriteGraphics( Graphics::SpriteSheet::LVTILESET_CITY, 2*8, 0 ),
            { new BlockComponentSolid() }
        ));
        block_types_.push_back( new BlockType
        (
            new SpriteGraphics( Graphics::SpriteSheet::LVTILESET_CITY, 27*8, 8 ),
            new SpriteGraphics( Graphics::SpriteSheet::LVTILESET_CITY, 28*8, 8 ),
            new SpriteGraphics( Graphics::SpriteSheet::LVTILESET_CITY, 27*8, 16 ),
            new SpriteGraphics( Graphics::SpriteSheet::LVTILESET_CITY, 28*8, 16 ),
            { new BlockComponentSolid() }
        ));
        block_types_.push_back( new BlockType
        (
            new SpriteGraphics( Graphics::SpriteSheet::LVTILESET_CITY, 0, 16 ),
            new SpriteGraphics( Graphics::SpriteSheet::LVTILESET_CITY, 8, 16 ),
            new SpriteGraphics( Graphics::SpriteSheet::LVTILESET_CITY, 0, 24 ),
            new SpriteGraphics( Graphics::SpriteSheet::LVTILESET_CITY, 8, 24 ),
            { new BlockComponentGoal() }, { { new BlockConditionCollideAny(), new BlockConditionHero() } }
        ));
        block_types_.push_back( new BlockType
        (
            new SpriteGraphics( Graphics::SpriteSheet::LVTILESET_CITY, 31*8, 16 ),
            new SpriteGraphics( Graphics::SpriteSheet::LVTILESET_CITY, 31*8, 16, true ),
            new SpriteGraphics( Graphics::SpriteSheet::LVTILESET_CITY, 31*8, 16 ),
            new SpriteGraphics( Graphics::SpriteSheet::LVTILESET_CITY, 31*8, 16, true ),
            {}//{}//BlockType::BlockAction::__NULL
        ));
        block_types_.push_back( new BlockType
        (
            new SpriteGraphics( Graphics::SpriteSheet::LVTILESET_CITY, 40, 24 ),
            new SpriteGraphics( Graphics::SpriteSheet::LVTILESET_CITY, 40, 24 ),
            new SpriteGraphics( Graphics::SpriteSheet::LVTILESET_CITY, 40, 24 ),
            new SpriteGraphics( Graphics::SpriteSheet::LVTILESET_CITY, 40, 24 ),
            { new BlockComponentSolid() }
        ));
        block_types_.push_back( new BlockType
        (
            new SpriteGraphics( Graphics::SpriteSheet::LVTILESET_CITY, 64, 24, 0, 0, 90 ),
            new SpriteGraphics( Graphics::SpriteSheet::LVTILESET_CITY, 64, 16, 0, 0, 90 ),
            new SpriteGraphics( Graphics::SpriteSheet::LVTILESET_CITY, 64, 24, 0, 0, 90 ),
            new SpriteGraphics( Graphics::SpriteSheet::LVTILESET_CITY, 64, 16, 0, 0, 90 ),
            { new BlockComponentSolid() }
        ));
        block_types_.push_back( new BlockType
        (
            new SpriteGraphics( Graphics::SpriteSheet::LVTILESET_CITY, 16, 16 ),
            new SpriteGraphics( Graphics::SpriteSheet::LVTILESET_CITY, 16, 16 ),
            new SpriteGraphics( Graphics::SpriteSheet::LVTILESET_CITY, 16, 16 ),
            new SpriteGraphics( Graphics::SpriteSheet::LVTILESET_CITY, 16, 16 ),
            { new BlockComponentSolid() }
        ));
        block_types_.push_back( new BlockType
        (
            new SpriteGraphics( Graphics::SpriteSheet::LVTILESET_CITY, 16, 8, false, false, 0, true ),
            new SpriteGraphics( Graphics::SpriteSheet::LVTILESET_CITY, 16, 8, false, false, 0, true ),
            new SpriteGraphics( Graphics::SpriteSheet::LVTILESET_CITY, 16, 8, false, false, 0, true ),
            new SpriteGraphics( Graphics::SpriteSheet::LVTILESET_CITY, 16, 8, false, false, 0, true ),
            { new BlockComponentWarp() }, { { new BlockConditionCollideAny(), new BlockConditionHero() } }
        ));
        block_types_.push_back( new BlockType
        (
            new SpriteGraphics( Graphics::SpriteSheet::LVTILESET_CITY, 16, 8 ),
            new SpriteGraphics( Graphics::SpriteSheet::LVTILESET_CITY, 16, 8 ),
            new SpriteGraphics( Graphics::SpriteSheet::LVTILESET_CITY, 16, 8 ),
            new SpriteGraphics( Graphics::SpriteSheet::LVTILESET_CITY, 16, 8 ),
            {}//BlockType::BlockAction::WARP2
        ));
        block_types_.push_back( new BlockType
        (
            new SpriteGraphics( Graphics::SpriteSheet::LVTILESET_CITY, 16, 8 ),
            new SpriteGraphics( Graphics::SpriteSheet::LVTILESET_CITY, 16, 8 ),
            new SpriteGraphics( Graphics::SpriteSheet::LVTILESET_CITY, 16, 8 ),
            new SpriteGraphics( Graphics::SpriteSheet::LVTILESET_CITY, 16, 8 ),
            {}//BlockType::BlockAction::WARP3
        ));
        block_types_.push_back( new BlockType
        (
            new SpriteGraphics( Graphics::SpriteSheet::LVTILESET_CITY, 16, 8 ),
            new SpriteGraphics( Graphics::SpriteSheet::LVTILESET_CITY, 16, 8 ),
            new SpriteGraphics( Graphics::SpriteSheet::LVTILESET_CITY, 16, 8 ),
            new SpriteGraphics( Graphics::SpriteSheet::LVTILESET_CITY, 16, 8 ),
            {}//BlockType::BlockAction::WARP4
        ));
        block_types_.push_back( new BlockType
        (
            new SpriteGraphics( Graphics::SpriteSheet::LVTILESET_CITY, 16, 8 ),
            new SpriteGraphics( Graphics::SpriteSheet::LVTILESET_CITY, 16, 8 ),
            new SpriteGraphics( Graphics::SpriteSheet::LVTILESET_CITY, 16, 8 ),
            new SpriteGraphics( Graphics::SpriteSheet::LVTILESET_CITY, 16, 8 ),
            {}//BlockType::BlockAction::WARP5
        ));
        block_types_.push_back( new BlockType
        (
            new SpriteGraphics( Graphics::SpriteSheet::LVTILESET_CITY, 24, 16 ),
            new SpriteGraphics( Graphics::SpriteSheet::LVTILESET_CITY, 16, 16 ),
            new SpriteGraphics( Graphics::SpriteSheet::LVTILESET_CITY, 24, 24 ),
            new SpriteGraphics( Graphics::SpriteSheet::LVTILESET_CITY, 32, 24 ),
            { new BlockComponentSolid() }
        ));
        block_types_.push_back( new BlockType
        (
            new SpriteGraphics( Graphics::SpriteSheet::LVTILESET_CITY, 16, 16 ),
            new SpriteGraphics( Graphics::SpriteSheet::LVTILESET_CITY, 6*8, 16 ),
            new SpriteGraphics( Graphics::SpriteSheet::LVTILESET_CITY, 16, 24 ),
            new SpriteGraphics( Graphics::SpriteSheet::LVTILESET_CITY, 6*8, 24 ),
            { new BlockComponentSolid() }
        ));
        block_types_.push_back( new BlockType
        (
            new SpriteGraphics( Graphics::SpriteSheet::LVTILESET_CITY, 32, 16 ),
            new SpriteGraphics( Graphics::SpriteSheet::LVTILESET_CITY, 16, 16 ),
            new SpriteGraphics( Graphics::SpriteSheet::LVTILESET_CITY, 32, 16 ),
            new SpriteGraphics( Graphics::SpriteSheet::LVTILESET_CITY, 16, 16 ),
            { new BlockComponentSolid() }
        ));
        block_types_.push_back( new BlockType
        (
            new SpriteGraphics( Graphics::SpriteSheet::LVTILESET_CITY, 16, 16 ),
            new SpriteGraphics( Graphics::SpriteSheet::LVTILESET_CITY, 40, 16 ),
            new SpriteGraphics( Graphics::SpriteSheet::LVTILESET_CITY, 16, 16 ),
            new SpriteGraphics( Graphics::SpriteSheet::LVTILESET_CITY, 40, 16 ),
            { new BlockComponentSolid() }
        ));
        block_types_.push_back( nullptr );
        block_types_.push_back( nullptr );
        block_types_.push_back( new BlockType
        (
            new SpriteGraphics( Graphics::SpriteSheet::LVTILESET_CITY, 7*8, 16 ),
            new SpriteGraphics( Graphics::SpriteSheet::LVTILESET_CITY, 64, 16 ),
            new SpriteGraphics( Graphics::SpriteSheet::LVTILESET_CITY, 7*8, 24 ),
            new SpriteGraphics( Graphics::SpriteSheet::LVTILESET_CITY, 64, 24 ),
            { new BlockComponentSolid() }
        ));
        block_types_.push_back( new BlockType
        (
            new SpriteGraphics( Graphics::SpriteSheet::LVTILESET_CITY, 64, 16 ),
            new SpriteGraphics( Graphics::SpriteSheet::LVTILESET_CITY, 64, 16 ),
            new SpriteGraphics( Graphics::SpriteSheet::LVTILESET_CITY, 64, 24 ),
            new SpriteGraphics( Graphics::SpriteSheet::LVTILESET_CITY, 64, 24 ),
            { new BlockComponentSolid() }
        ));
        block_types_.push_back( new BlockType
        (
            new SpriteGraphics( Graphics::SpriteSheet::LVTILESET_CITY, 64, 16 ),
            new SpriteGraphics( Graphics::SpriteSheet::LVTILESET_CITY, 72, 16 ),
            new SpriteGraphics( Graphics::SpriteSheet::LVTILESET_CITY, 64, 24 ),
            new SpriteGraphics( Graphics::SpriteSheet::LVTILESET_CITY, 72, 24 ),
            { new BlockComponentSolid() }
        ));
        block_types_.push_back( new BlockType
        (
            new SpriteGraphics( Graphics::SpriteSheet::LVTILESET_CITY, 64, 24, 0, 0, 90 ),
            new SpriteGraphics( Graphics::SpriteSheet::LVTILESET_CITY, 64, 16, 0, 0, 90 ),
            new SpriteGraphics( Graphics::SpriteSheet::LVTILESET_CITY, 72, 24, 0, 0, 90 ),
            new SpriteGraphics( Graphics::SpriteSheet::LVTILESET_CITY, 72, 16, 0, 0, 90 ),
            { new BlockComponentSolid() }
        ));
        block_types_.push_back( new BlockType
        (
            new SpriteGraphics( Graphics::SpriteSheet::LVTILESET_CITY, 16, 16 ),
            new SpriteGraphics( Graphics::SpriteSheet::LVTILESET_CITY, 16, 16 ),
            new SpriteGraphics( Graphics::SpriteSheet::LVTILESET_CITY, 16, 24 ),
            new SpriteGraphics( Graphics::SpriteSheet::LVTILESET_CITY, 32, 24 ),
            { new BlockComponentSolid() }
        ));
        block_types_.push_back( new BlockType
        (
            new SpriteGraphics( Graphics::SpriteSheet::LVTILESET_CITY, 64, 8 ),
            new SpriteGraphics( Graphics::SpriteSheet::LVTILESET_CITY, 72, 8 ),
            new SpriteGraphics( Graphics::SpriteSheet::LVTILESET_CITY, 64, 8 ),
            new SpriteGraphics( Graphics::SpriteSheet::LVTILESET_CITY, 72, 8 ),
            { new BlockComponentWarp(), new BlockComponentClimbable() }, { { new BlockConditionCollideAny(), new BlockConditionHero() }, { new BlockConditionCollideAny() } }
        ));
        block_types_.push_back( new BlockType
        (
            new SpriteGraphics( Graphics::SpriteSheet::LVTILESET_CITY, 64, 8 ),
            new SpriteGraphics( Graphics::SpriteSheet::LVTILESET_CITY, 72, 8 ),
            new SpriteGraphics( Graphics::SpriteSheet::LVTILESET_CITY, 64, 8 ),
            new SpriteGraphics( Graphics::SpriteSheet::LVTILESET_CITY, 72, 8 ),
            {}//BlockType::BlockAction::WARP2
        ));
        block_types_.push_back( new BlockType
        (
            new SpriteGraphics( Graphics::SpriteSheet::LVTILESET_CITY, 64, 8 ),
            new SpriteGraphics( Graphics::SpriteSheet::LVTILESET_CITY, 72, 8 ),
            new SpriteGraphics( Graphics::SpriteSheet::LVTILESET_CITY, 64, 8 ),
            new SpriteGraphics( Graphics::SpriteSheet::LVTILESET_CITY, 72, 8 ),
            {}//BlockType::BlockAction::WARP3
        ));
        block_types_.push_back( new BlockType
        (
            new SpriteGraphics( Graphics::SpriteSheet::LVTILESET_CITY, 64, 8 ),
            new SpriteGraphics( Graphics::SpriteSheet::LVTILESET_CITY, 72, 8 ),
            new SpriteGraphics( Graphics::SpriteSheet::LVTILESET_CITY, 64, 8 ),
            new SpriteGraphics( Graphics::SpriteSheet::LVTILESET_CITY, 72, 8 ),
            {}//BlockType::BlockAction::WARP4
        ));
        block_types_.push_back( new BlockType
        (
            new SpriteGraphics( Graphics::SpriteSheet::LVTILESET_CITY, 64, 8 ),
            new SpriteGraphics( Graphics::SpriteSheet::LVTILESET_CITY, 72, 8 ),
            new SpriteGraphics( Graphics::SpriteSheet::LVTILESET_CITY, 64, 8 ),
            new SpriteGraphics( Graphics::SpriteSheet::LVTILESET_CITY, 72, 8 ),
            {}//BlockType::BlockAction::WARP5
        ));
    };

    BlockSystem::~BlockSystem()
    {
        clearBlockTypes();
    };

    void BlockSystem::update( Camera& camera, Input& input )
    {
    };

    void BlockSystem::render( Graphics& graphics, Camera& camera, bool priority )
    {
        for ( int i = 0; i < blocks_used_; ++i )
        {
            blocks_[i].render( graphics, camera, priority );
        }
    };

    void BlockSystem::clearBlocks()
    {
        for (int i = 0; i < blocks_used_; ++i)
        {
            blocks_[i].neutralize();
        }

        blocks_used_ = 0;
    };

    void BlockSystem::addBlock( int x, int y, BlockType* type, int location )
    {
        if ( blocks_used_ < LIMIT )
        {
            blocks_[ blocks_used_ ].setBlock( x, y, type, location );
            ++blocks_used_;
        }
    };

    void BlockSystem::interact( Sprite& sprite, Level& level, EventSystem& events, Inventory& inventory, Camera& camera )
    {
        for (int i = 0; i < blocks_used_; ++i)
        {
            blocks_[ i ].interact( sprite, level, events, inventory, camera );
        }
    };

    void BlockSystem::blocksFromMap( Map& lvmap, Camera& camera )
    {
        if ( camera.changed() || lvmap.changed() )
        {
            clearBlocks();

            SDL_Rect bc = { 0, 0, 16000, 16000 };

            for ( int i = 0; i < lvmap.blocksSize(); ++i )
            {
                bc.x = Unit::BlocksToSubPixels( lvmap.mapX( i ) );
                bc.y = Unit::BlocksToSubPixels( lvmap.mapY( i ) );

                if ( camera.onscreen( bc, 16 ) )
                {
                    const int type = lvmap.block( i ) - 1;

                    if ( type > -1 )
                    {
                        addBlock( bc.x, bc.y, blockType( type ), i );
                    }
                }
            }
        }

        lvmap.update();
    };

    BlockType* BlockSystem::blockType( int n ) const
    {
        if ( n > -1 && n < block_types_.size() )
        {
            return block_types_[ n ];
        }
        else
        {
            return nullptr;
        }
    };

    void BlockSystem::clearBlockTypes()
    {
        for ( int i = 0; i < block_types_.size(); ++i )
        {
            delete block_types_[ i ];
            block_types_[ i ] = nullptr;
        }
    };
