


// Name
//===================================
//
// BlockType
//

#ifndef BLOCK_TYPE_H
#define BLOCK_TYPE_H


// FORWARD DECLARATIONS
//===================================

    class Block;
    class BlockComponent;
    class BlockCondition;
    class Collision;
    class EventSystem;
    class Inventory;
    class GameState;
    class Level;
    class Sprite;
    class SpriteGraphics;


// DEPENDENCIES
//===================================

    #include <map>
    #include "object.h"
    #include <vector>


// CLASS
//===================================

    class BlockType
    {
        public:
            BlockType
            (
                SpriteGraphics* g1 = nullptr,
                SpriteGraphics* g2 = nullptr,
                SpriteGraphics* g3 = nullptr,
                SpriteGraphics* g4 = nullptr,
                std::vector<BlockComponent*> components = {},
                std::vector<std::vector<BlockCondition*>> conditions = {}
            );

            ~BlockType();

            void interact( Collision& collision, Sprite& sprite, Block& block, Level& level, EventSystem& events, Inventory& inventory, Camera& camera );
            void render( Graphics& graphics, Camera& camera, Block& block, bool priority );


        private:
            static const int BLOCK_SIZE = 16;
            static const int MINI_BLOCK_SIZE = 8;
            static const int NUM_O_MINI_BLOCKS = 4;

            SpriteGraphics* graphics_[ NUM_O_MINI_BLOCKS ] = { nullptr };
            std::vector<std::vector<BlockCondition*>> conditions_;
            std::vector<BlockComponent*> components_;
    };

#endif // BLOCK_TYPE_H
