

// Name
//===================================
//
// Clock
//


// DEPENDENCIES
//===================================

    #include "clock.h"
    #include <cmath>
    #include "text.h"


// STATIC PROPERTIES
//===================================

// METHODS
//===================================

    Clock::Clock
    (
        int start_time,
        Direction::Vertical direction,
        int limit
    )
    :
        total_seconds_ ( start_time ),
        direction_ ( direction ),
        limit_ ( limit )
    {};

    Clock::~Clock() {};

    void Clock::update()
    {
        if ( timer_.hit() )
        {/*
            if ( direction_ == Direction::Vertical::DOWN )
            {
                --total_seconds_;

                if ( total_seconds_ < limit_ )
                    total_seconds_ = limit_;
            }
            else
            {*/
                ++total_seconds_;

                if ( total_seconds_ > limit_ )
                    total_seconds_ = limit_;
            //}
        }

        timer_.update();
    };

    int Clock::secondsFromTotal() const
    {
        return secondsFromTotal( total_seconds_ );
    };

    int Clock::minutesFromTotalSeconds() const
    {
        return minutesFromTotalSeconds( total_seconds_ );
    };

    int Clock::secondsFromTotal( int total_seconds )
    {
        return total_seconds % SECONDS_PER_MINUTE;
    };

    int Clock::minutesFromTotalSeconds( int total_seconds )
    {
        return floor( total_seconds / SECONDS_PER_MINUTE );
    };

    int Clock::totalSeconds() const
    {
        return total_seconds_;
    };

    bool Clock::hitLimit() const
    {
        return total_seconds_ >= limit_;
    };

    void Clock::renderTime( Graphics& graphics, int x, int y, int total_seconds )
    {
        Text::renderText( graphics, Text::timeToString( secondsFromTotal( total_seconds ), minutesFromTotalSeconds( total_seconds ) ), x, y );
    }

    std::string Clock::timeToString( int total_seconds )
    {
        return Text::timeToString( secondsFromTotal( total_seconds ), minutesFromTotalSeconds( total_seconds ) );
    };

    void Clock::render( Graphics& graphics, int x, int y )
    {
        if ( direction_ == Direction::Vertical::DOWN )
        {
            renderTime( graphics, x, y, limit_ - total_seconds_ );
        }
        else
        {
            renderTime( graphics, x, y, total_seconds_ );
        }
    };
