
// Name
//===================================
//
// Graphics
//

#ifndef GRAPHICS_H
#define GRAPHICS_H


// FORWARD DECLARATIONS
//===================================


// DEPENDENCIES
//===================================

    #include "bosk_sdl2.h"
    #include <iostream>
    #include "palette.h"
    #include <SDL2/SDL.h>
    #include "timers/timer_repeat.h"


// CLASS
//===================================

    class Graphics
    {
        public:
            enum SpriteSheet
            {
                CHARSET,
                BOSKO_LOGO,
                LVTILESET_UNIVERSAL,
                LVTILESET_CITY,
                LVSPRITES_AUTUMN,
                LVSPRITES_HYDRANT,
                LVBG_CITY,
                LVBG_WOODS,
                LVBG_WOODS2,
                LVBG_MOUNTAIN,
                LVBG_SMOG,
                LVBG_RAIN,
                LVSPRITES_SAW,
                LVSPRITES_EYEGUARD,
                LIMIT
            };

            Graphics();
            ~Graphics();
            const std::string imgAddress( const std::string& relative_path ) const;
            SDL_Texture* loadTexture( SpriteSheet sheet );

            void colorCanvas();
            void clearScreen();
            void screenBorders();
            void presentScreen();
            void clearTextures();
            void renderObject( SpriteSheet sheet, SDL_Rect source, SDL_Rect dest, SDL_RendererFlip flip = SDL_FLIP_NONE, double rotation = 0 );
            void renderRect( SDL_Rect box, int color = 1, int alpha = 255 );

            void newPalette( Palette::PaletteSet palette );
            void reloadTextures();
            static void update();
            static bool nextFrame();

        private:
            const char* graphic_addresses_[ SpriteSheet::LIMIT ] =
            {
                "charset.png",
                "bosko_logo.png",
                "tileset_level_universal.png",
                "tileset_level_city.png",
                "sprites_level_autumn.png",
                "sprites_level_hydrant.png",
                "bg_level_city.png",
                "bg_level_woods.png",
                "bg_level_woods2.png",
                "bg_level_mountain.png",
                "bg_level_smog.png",
                "bg_level_rain.png",
                "sprites_level_saw.png",
                "sprites_level_eyeguard.png"
            };
            //sdl2::SDLTexture textures_[SpriteSheet::LIMIT] = { nullptr };
            SDL_Texture* textures_[SpriteSheet::LIMIT] = { nullptr };
            SDL_Surface* surfaces_[SpriteSheet::LIMIT] = { nullptr };
            sdl2::SDLRenderer renderer_ = nullptr; // Special init & destruct ptr.
            sdl2::SDLWindow   window_ = nullptr; // Special init & destruct ptr.
            SDL_Rect screen_;
            static TimerRepeat animation_frame_;
            Palette* palette_ = new Palette();

            int magnification_ = 1;
            int left_edge_ = 0;
            int top_edge_ = 0;

            SDL_Rect top_bar_ = { 0, 0, 0, 0 };
            SDL_Rect bottom_bar_ = { 0, 0, 0, 0 };
            SDL_Rect left_bar_ = { 0, 0, 0, 0 };
            SDL_Rect right_bar_ = { 0, 0, 0, 0 };

            int magnified( int n ) const;
            void adjustMagnification();
            void adjustScreen();

            bool createRenderer();
            bool createWindow();
            void deletePalette();

            SDL_Rect sourceRelativeToScreen( SDL_Rect source ) const;
    };

#endif // GRAPHICS_H

