


// Name
//===================================
//
// HydrantSprite
//


// DEPENDENCIES
//===================================

    #include "collision.h"
    #include "hydrant_sprite.h"
    #include "hydrant_graphics.h"


// STATIC PROPERTIES
//===================================


// METHODS
//===================================

    HydrantSprite::HydrantSprite( int x, int y )
    :
        Sprite( new HydrantGraphics(), x, y, 14, 14, SpriteType::ENEMY, 100, 1500, 3000 )
    {};

    HydrantSprite::~HydrantSprite() {};

    void HydrantSprite::customUpdate( Input& input, Camera& camera, Map& lvmap, Game& game, EventSystem& events )
    {
        if ( isDead() )
            interact_ = false;

        if ( awake_ && delay_.done() )
        {
            if ( onGround() )
                jump();
        }
        else if ( delay_.counter() > 24 )
        {
            graphics_->current_frame_x_ = 32;
        }
        else if ( delay_.counter() > 8 )
        {
            graphics_->current_frame_x_ = 16;
        }

        if ( delay_.counter() % 4 == 1 )
        {
            graphics_->current_frame_y_ = 16;
        }
        else
        {
            graphics_->current_frame_y_ = 0;
        }
    };

    void HydrantSprite::customInteract( Collision& my_collision, Collision& their_collision, Sprite& them )
    {
        if ( them.hasType( Sprite::SpriteType::HERO ) )
        {
            if ( awake_ )
            {
                if ( delay_.done() )
                {
                    if ( them.xSubPixels() > hitboxRight() )
                        moveRight();
                    else
                        moveLeft();

                    if ( their_collision.collideAny() && !their_collision.collideBottom() )
                        them.hurt();
                    else if ( their_collision.collideBottom() )
                    {
                        kill();
                        them.collideStopYBottom( their_collision.overlapYBottom() );
                        them.jump();
                    }
                }
                else
                {
                    delay_.update();
                }
            }
            else
            {
                if ( them.hitboxRight() > xSubPixels() - 40000 && them.xSubPixels() < hitboxRight() + 40000 )
                {
                    awake_ = true;
                    delay_.start();
                }
            }

            if ( !delay_.done() )
            {
                them.collideStopAny( their_collision );
            }

        }
        else if ( them.hasType( Sprite::SpriteType::ENEMY ) )
        {
            collideStopAny( my_collision );
        }
    };
