


// Name
//===================================
//
// Level
//

#ifndef LEVEL_H
#define LEVEL_H


// FORWARD DECLARATIONS
//===================================

    class Camera;
    class GameState;
    class Goal;
    class Graphics;
    class Sprite;
    class SpriteSystem;


// DEPENDENCIES
//===================================

    #include "map.h"
    #include <vector>


// CLASS
//===================================

    class Level
    {
        public:
            enum LevelName
            {
                LV_CITY_1,
                LV_SEWER_1,
                LV_WOODS_1,
                LV_MOUNTAIN_1,
                LV_SAW,
                LV_WOODS_2,
                LV_CITY_2,
                LV_CITY_3,
                LV_CITY_4,
                LV_SNEAK,
                LV_LIMIT
            };

            Level( LevelName level = LevelName::LV_CITY_1 );
            ~Level();
            Map& currentMap();

            int entranceX() const;
            int entranceY() const;
            int cameraX() const;
            int cameraY() const;
            void warp( SpriteSystem& sprites, Camera& camera, GameState* state, Graphics& graphics );
            Goal* goal() const;
            static std::string NameOLevel( int n );
            std::string message() const;
            LevelName id() const;

        private:
            Level::LevelName id_;
            std::vector<Map> maps_;
            //std::vector<Warp> warps_;
            Goal* goal_ = nullptr;
            int current_map_ = 0;
            int default_entrance_x_ = 0;
            int default_entrance_y_ = 0;
            int entrance_x_ = 0;
            int entrance_y_ = 0;
            int camera_x_ = 0;
            int camera_y_ = 0;
            std::string message_ = "";

            bool testCurrentMap();
            //bool testWarp( int n ) const;
            void reset();
            void changeGoal( Goal* goal = nullptr );
            void deleteGoal();
    };

#endif // LEVEL_H

