


// Name
//===================================
//
// LevelSelectState
//


// DEPENDENCIES
//===================================

    #include "game.h"
    #include "input.h"
    #include "level_state.h"
    #include "level_select_state.h"
    #include "title_state.h"


// STATIC PROPERTIES
//===================================


// METHODS
//===================================

    LevelSelectState::LevelSelectState( EventSystem* events, Inventory* inventory, Level::LevelName level )
    :
        prev_level_ ( level ),
        selection_ ( Counter( level, Level::LevelName::LV_LIMIT-1, 0 ) ),
        GameState ( StateID::LEVEL_SELECT_STATE )
    {
        if ( events != nullptr )
        {
            events_ = events;
        }
        else
            events_ = new EventSystem();

        if ( inventory != nullptr )
            inventory_ = inventory;
        else
            inventory_ = new Inventory();
    };

    LevelSelectState::~LevelSelectState() {};

    void LevelSelectState::update( Game& game, Input& input, Graphics& graphics )
    {
        inventory_->updateLevelSelect();

        highlight_rect_.y = START_Y + ( 8 * selection_.value() );

        if ( delay_ <= 0 )
        {
            if ( input.held( Input::Action::MOVE_UP ) )
            {
                --selection_;
                delay_ = delay_length_;
            }
            if ( input.held( Input::Action::MOVE_DOWN ) )
            {
                ++selection_;
                delay_ = delay_length_;
            }
        }
        else
        {
            --delay_;
        }

        if ( input.held( Input::Action::MOVE_UP ) || input.held( Input::Action::MOVE_DOWN ) )
            delay_length_ = 4;
        else
            delay_length_ = 8;

        if ( input.pressed( Input::Action::CONFIRM ) )
        {
            game.changeState( std::unique_ptr<GameState> ( new LevelState( *events_, *inventory_, (Level::LevelName)selection_.value() ) ) );
        }
        else if ( input.pressed( Input::Action::MENU ) )
        {
            game.changeState( std::unique_ptr<GameState> ( new TitleState() ) );
        }
    };

    void LevelSelectState::stateRender( Graphics& graphics )
    {
        title_.render( graphics );
        graphics.renderRect( highlight_rect_, 6 );

        int i = 0;
        for ( int i = 0; i < levels_.size(); ++i )
        {
            Text::FontShade shade = Text::FontShade::BLACK;

            if ( i == selection_.value() )
                shade = Text::FontShade::WHITE;

            levels_[ i ].render( graphics, nullptr, shade );

            if ( i < levels_gem_scores_.size() )
                levels_gem_scores_[ i ].render( graphics, nullptr, shade );

            if ( i < levels_time_scores_.size() )
                levels_time_scores_[ i ].render( graphics, nullptr, shade );
        }

        for ( int j = 0; j < win_icon_rects_.size(); ++j )
        {
            win_icon_gfx_.render( graphics, win_icon_rects_[ j ] );
        }

        if ( inventory_ != nullptr )
            inventory_->renderLevelSelect( graphics );
    };

    void LevelSelectState::init( Game& game, Graphics& graphics )
    {
        if ( inventory_ != nullptr )
        {
            if ( events_->won_ )
            {
                if ( prev_level_ < Level::LevelName::LV_LIMIT && prev_level_ >= 0 )
                {
                    inventory_->victories_[ (int)prev_level_ ] = true;

                    if ( inventory_->funds() > inventory_->gem_scores_[ (int)prev_level_ ] )
                        inventory_->gem_scores_[ (int)prev_level_ ] = inventory_->funds();

                    if ( inventory_->clockTime() < inventory_->time_scores_[ (int)prev_level_ ] || inventory_->time_scores_[ (int)prev_level_ ] <= 0 )
                        inventory_->time_scores_[ (int)prev_level_ ] = inventory_->clockTime();
                }
            }
        }

        int X = 24;

        for ( int i = 0; i < Level::LevelName::LV_LIMIT; ++i )
        {
            levels_.push_back( Text( Level::NameOLevel( i ), X+8, START_Y+(8*i) ) );
            levels_gem_scores_.push_back( Text( inventory_->gemScore( (Level::LevelName)i ), X+(23*8), START_Y+(8*i) ) );
            levels_time_scores_.push_back( Text( inventory_->timeScore( (Level::LevelName)i ), X+(30*8), START_Y+(8*i) ) );

            if ( inventory_ != nullptr )
            {
                if ( inventory_->victory( (Level::LevelName)i ) )
                {
                    win_icon_rects_.push_back( { X, START_Y+(8*i), 8, 8 } );
                }
            }
        }

        inventory_->initLevelSelect( events_->won_ );
        events_->reset();
    };

    void LevelSelectState::backFromPop( Game& game, Graphics& graphics ) {};
