


// Name
//===================================
//
// LevelState
//


// DEPENDENCIES
//===================================

    #include "background.h"
    #include "game.h"
    #include "goal.h"
    #include "level_state.h"
    #include "message_state.h"
    #include "level_message_state.h"
    #include "level_select_state.h"


// STATIC PROPERTIES
//===================================


// METHODS
//===================================

    LevelState::LevelState( EventSystem& events, Inventory& inventory, Level::LevelName lvname )
    :
        events_ ( events ),
        inventory_ ( inventory ),
        current_level_ ( lvname ),
        level_ ( Level( lvname ) ),
        GameState ( StateID::LEVEL_STATE )
    {
    };
/*
    LevelState::LevelState( bool show_message, int lvname )
    :
        LevelState( show_message, (Level::LevelName)lvname ),
        GameState ( StateID::LEVEL_STATE )
    {
    };*/

    LevelState::~LevelState() {};

    void LevelState::update( Game& game, Input& input, Graphics& graphics )
    {
        blocks_.blocksFromMap( level_.currentMap(), camera_ );
        camera_.adjust( sprites_.hero(), level_.currentMap().widthPixels(), level_.currentMap().heightPixels() );
        sprites_.update( input, camera_, level_.currentMap(), game, events_ );
        sprites_.interact( blocks_, level_, events_, inventory_, camera_ );
        sprites_.spriteInteraction( camera_ );
        inventory_.update();

        if ( level_.goal() != nullptr )
            level_.goal()->update( sprites_.hero(), inventory_, events_ );

        if ( sprites_.hero().hp() < inventory_.hearts() )
            inventory_.loseHeart();
        if ( sprites_.hero().hp() > inventory_.hearts() )
            inventory_.gainHeart();

        if ( events_.message_lock_ && !events_.message_ )
            events_.message_lock_ = false;

        if ( events_.message_ && !events_.message_lock_ )
        {
            game.pushState( std::unique_ptr<GameState> ( new LevelMessageState( level_.currentMap().palette(), level_.message() ) ) );
            events_.message_lock_ = true;
        }
        events_.message_ = false;

        if ( events_.change_map_ )
        {
            level_.warp( sprites_, camera_, this, graphics );
            events_.change_map_ = false;
        }

        if ( events_.failed_ )
        {
            game.pushState( std::unique_ptr<GameState> ( new MessageState( "¡Failure!", false, std::unique_ptr<GameState> ( new LevelSelectState( &events_, &inventory_, level_.id() ) ), false, { Palette::PaletteType::MOUNTAIN_RED, 2 }, Text::FontShade::DARK_GRAY ) ) );
        }
        else if ( events_.won_ )
        {
            game.pushState( std::unique_ptr<GameState> ( new MessageState( "¡Success!", false, std::unique_ptr<GameState> ( new LevelSelectState( &events_, &inventory_, level_.id() ) ), false, { Palette::PaletteType::CLASSIC_GREEN, 2 }, Text::FontShade::DARK_GRAY ) ) );
        }
        else if ( events_.quit_level_ )
        {
            game.changeState( std::unique_ptr<GameState> ( new LevelSelectState( &events_, &inventory_, level_.id() ) ) );
        }
    };

    void LevelState::stateRender( Graphics& graphics )
    {
        level_.currentMap().renderBG( graphics, camera_ );
        blocks_.render( graphics, camera_, false );
        sprites_.render( graphics, camera_ );
        blocks_.render( graphics, camera_, true );
        level_.currentMap().renderFG( graphics, camera_ );
        inventory_.render( graphics );
    };

    void LevelState::init( Game& game, Graphics& graphics )
    {
        newPalette( graphics, level_.currentMap().palette() );
        if ( level_.goal() != nullptr )
        {
            game.pushState( std::unique_ptr<GameState> ( new MessageState( level_.goal()->message(), true, nullptr, false, { Palette::PaletteType::SUNNY_YELLOW, 2 }, Text::FontShade::DARK_GRAY ) ) );
        }

        if ( level_.goal() != nullptr )
        {
            if ( level_.goal()->type() == Goal::GoalType::TIMED )
            {
                inventory_.init( new Clock( 0, Direction::Vertical::DOWN, level_.goal()->value() ) );
            }
            else
            {
                inventory_.init();
            }
        }
        else
        {
            inventory_.init();
        }

        sprites_.reset( level_.currentMap() );

    };


    void LevelState::backFromPop( Game& game, Graphics& graphics ) {};
