
// Name
//===================================
//
// Map
//


// DEPENDENCIES
//===================================

    #include <algorithm>
    #include <iostream>
    #include "map.h"
    #include "unit.h"


// STATIC PROPERTIES
//===================================

// METHODS
//===================================

    Map::Map
    (
        std::vector<int> blocks,
        std::vector<int> sprites,
        int width,
        int height,
        Palette::PaletteSet palette,
        std::vector<Background> backgrounds,
        std::vector<Warp> warps,
        std::vector<Background> foregrounds
    )
    :
        blocks_ ( blocks ),
        sprites_ ( sprites ),
        width_ ( width ),
        height_ ( height ),
        palette_ ( palette ),
        backgrounds_ ( backgrounds ),
        warps_ ( warps ),
        foregrounds_ ( foregrounds )
    {
    };

    Map::~Map()
    {
    };

    int Map::widthBlocks() const
    {
        return width_;
    };

    int Map::heightBlocks() const
    {
        return height_;
    };

    int Map::widthPixels() const
    {
        return Unit::BlocksToPixels( width_ );
    };

    int Map::heightPixels() const
    {
        return Unit::BlocksToPixels( height_ );
    };

    int Map::blocksSize() const
    {
        return std::min( widthBlocks() * heightBlocks(), (int)blocks_.size() );
    };

    int Map::spritesSize() const
    {
        return std::min( widthBlocks() * heightBlocks(), (int)sprites_.size() );
    };

    int Map::block( int n ) const
    {
        return blocks_[ n ];
    };

    int Map::sprite( int n ) const
    {
        return sprites_[ n ];
    };

    int Map::mapX( int n ) const
    {
        return n % widthBlocks();
    };

    int Map::mapY( int n ) const
    {
        return floor( n / widthBlocks() );
    };

    const bool Map::changed() const
    {
        return changed_;
    };

    void Map::changeBlock( int where, int value )
    {
        if ( inBounds( where ) )
        {
            blocks_[ where ] = value;
        }
        changed_ = true;
    };

    void Map::deleteBlock( int where )
    {
        changeBlock( where, 0 );
    };

    bool Map::inBounds( int n ) const
    {
        return n > 0 && n < blocks_.size();
    };

    void Map::update()
    {
        changed_ = false;
    };

    Palette::PaletteSet Map::palette() const
    {
        return palette_;
    };

    void Map::renderBG( Graphics& graphics, Camera& camera )
    {
        for ( int i = 0; i < backgrounds_.size(); ++i )
        {
            backgrounds_[ i ].render( graphics, camera );
        }
    };

    void Map::renderFG( Graphics& graphics, Camera& camera )
    {
        for ( int i = 0; i < foregrounds_.size(); ++i )
        {
            foregrounds_[ i ].render( graphics, camera );
        }
    };

    Warp* Map::getWarp( int x_sub_pixels, int y_sub_pixels )
    {
        for ( int i = 0; i < warps_.size(); ++i )
        {
            if ( warps_[ i ].inInterval( x_sub_pixels, y_sub_pixels ) )
                return &warps_[ i ];
        }

        return nullptr;
    };
