

// Name
//===================================
//
// Object
//


// DEPENDENCIES
//===================================

    #include "collision.h"
    #include "object.h"
    #include "unit.h"
    #include <iostream>


// STATIC PROPERTIES
//===================================

// METHODS
//===================================

    Object::Object
    (
        int x,
        int y,
        int width,
        int height
    )
    :
        original_height_ ( height ),
        original_width_ ( width ),
        original_x_ ( x ),
        original_y_ ( y ),
        hit_box_ ( { Unit::PixelsToSubPixels( x ), Unit::PixelsToSubPixels( y ), Unit::PixelsToSubPixels( width ), Unit::PixelsToSubPixels( height ) } )
    {};

    Object::~Object() {};

    int Object::heightPixels() const
    {
        return Unit::SubPixelsToPixels( hit_box_.h );
    };

    int Object::widthPixels() const
    {
        return Unit::SubPixelsToPixels( hit_box_.w );
    };

    int Object::heightSubPixels() const
    {
        return hit_box_.h;
    };

    int Object::widthSubPixels() const
    {
        return hit_box_.w;
    };

    int Object::xPixels() const
    {
        return Unit::SubPixelsToPixels( hit_box_.x );
    };

    int Object::yPixels() const
    {
        return Unit::SubPixelsToPixels( hit_box_.y );
    };

    int Object::xSubPixels() const
    {
        return hit_box_.x;
    };

    int Object::ySubPixels() const
    {
        return hit_box_.y;
    };

    int Object::hitboxHalfWidth() const
    {
        return round( hit_box_.w / 2 );
    };

    int Object::hitboxHalfHeight() const
    {
        return round( hit_box_.h / 2 );
    };

    int Object::hitboxCenterX() const
    {
        return xSubPixels() + hitboxHalfWidth();
    };

    int Object::hitboxCenterY() const
    {
        return ySubPixels() + hitboxHalfHeight();
    };

    int Object::hitboxLeft() const
    {
        return hit_box_.x;
    };

    int Object::hitboxRight() const
    {
        return hit_box_.x + hit_box_.w;
    };

    int Object::hitboxTop() const
    {
        return hit_box_.y;
    };

    int Object::hitboxBottom() const
    {
        return hit_box_.y + hit_box_.h;
    };

    int Object::hitboxPrevLeft() const
    {
        return xPrevSubPixels();
    };

    int Object::hitboxPrevRight() const
    {
        return xPrevSubPixels() + hit_box_.w;
    };

    int Object::hitboxPrevTop() const
    {
        return yPrevSubPixels();
    };

    int Object::hitboxPrevBottom() const
    {
        return yPrevSubPixels() + hit_box_.h;
    };

    const Collision Object::collide( Object* other ) const
    {
        int overlap_x_left   = 0;
        int overlap_x_right  = 0;
        int overlap_y_top    = 0;
        int overlap_y_bottom = 0;
/*
        double dx = hitboxCenterX() - other->hitboxCenterX();
        double dy = hitboxCenterY() - other->hitboxCenterY();

        double hx = ( other->hitboxHalfWidth() ) + ( hitboxHalfWidth() );
        double hy = ( other->hitboxHalfHeight() ) + ( hitboxHalfHeight() );

        int overlap_x = hx - abs(dx);
        int overlap_y = hy - abs(dy);

        bool theres_a_collision = abs(dx) < hx && abs(dy) < hy;

        if ( theres_a_collision )
        {
            if ( dx > 0 )
              overlap_x_left = overlap_x;
            else
              overlap_x_right = overlap_x;

            if ( dy > 0 )
              overlap_y_top = overlap_y;
            else
              overlap_y_bottom = overlap_y;
        }
*/

        //SDL_Rect their_hit_box = other->boundBox();
        if
        (
            hitboxLeft() + 4000 < other->hitboxRight() &&
            hitboxRight() - 4000 > other->hitboxLeft() &&
            hitboxTop() < other->hitboxBottom() &&
            hitboxBottom() > other->hitboxTop()
        )
        {
            if ( hitboxCenterY() > other->hitboxCenterY() )
                overlap_y_top = other->hitboxBottom() - hitboxTop();
            else
                overlap_y_bottom = hitboxBottom() - other->hitboxTop();
        }

        if
        (
            hitboxLeft() + 1000 < other->hitboxRight() &&
            hitboxRight() - 3000 > other->hitboxLeft() &&
            hitboxTop() < other->hitboxBottom() &&
            hitboxBottom() > other->hitboxTop()
        )
        {
            if ( hitboxCenterY() < other->hitboxCenterY() )
                overlap_y_bottom = hitboxBottom() - other->hitboxTop();
        }

        if
        (
            hitboxLeft() < other->hitboxRight() &&
            hitboxRight() > other->hitboxLeft() &&
            hitboxTop() < other->hitboxBottom() &&
            hitboxBottom() - 4000 > other->hitboxTop()
        )
        {
            if ( hitboxCenterX() < other->hitboxCenterX() )
                overlap_x_right = other->hitboxRight() - hitboxLeft();
            else if ( hitboxCenterX() > other->hitboxCenterX() )
                overlap_x_left = hitboxRight() - other->hitboxLeft();
        }

/*
        const int left_bottom_point_x = hit_box_.x + 2000;
        const int right_bottom_point_x = hit_box_.x + hit_box_.w - 2000;
        const int center_bottom_point_x = hit_box_.x + hitboxHalfHeight();
        const int bottom_points_y = hit_box_.y + hit_box_.h;

        const int target_downward_left_bound_x = other->hit_box_.x;
        const int target_downward_right_bound_x = other->hit_box_.x + other->hit_box_.w;
        const int target_downward_top_bound_y = other->hit_box_.y;
        const int target_downward_bottom_bound_y = other->hit_box_.y + other->hitboxHalfHeight();//other->hit_box_.h;

        const int top_points_y = hit_box_.y;
        const int target_upward_top_bound_y = other->hit_box_.y + other->hitboxHalfHeight();
        const int target_upward_bottom_bound_y = other->hit_box_.y + other->hit_box_.h;

        const int left_point_x = hit_box_.x;
        const int left_point_y_top = hit_box_.y;
        const int left_point_y_center = hit_box_.y + hitboxHalfHeight();
        const int left_point_y_bottom = hit_box_.y + hit_box_.h - 8000;

        const int left_target_bound_x_left = other->hit_box_.x;
        const int left_target_bound_x_right = other->hit_box_.x + other->hit_box_.w;
        const int left_target_bound_y_top = other->hit_box_.y;
        const int left_target_bound_y_bottom = other->hit_box_.y + other->hit_box_.h;

        const int right_point_x = hit_box_.x + hit_box_.w;

        const int right_target_bound_x_left = other->hit_box_.x;
        const int right_target_bound_x_right = other->hit_box_.x + other->hitboxHalfWidth();

        if
        (
            ( left_bottom_point_x > target_downward_left_bound_x && left_bottom_point_x < target_downward_right_bound_x ) ||
            ( right_bottom_point_x > target_downward_left_bound_x && right_bottom_point_x < target_downward_right_bound_x ) ||
            ( center_bottom_point_x > target_downward_left_bound_x && center_bottom_point_x < target_downward_right_bound_x )
        )
        {
            if
            (
                bottom_points_y > target_downward_top_bound_y &&
                bottom_points_y < target_downward_bottom_bound_y
            )
            {
                overlap_y_bottom = bottom_points_y - target_downward_top_bound_y;
            }
        }

        if
        (
            ( left_bottom_point_x > target_downward_left_bound_x && left_bottom_point_x < target_downward_right_bound_x ) ||
            ( right_bottom_point_x > target_downward_left_bound_x && right_bottom_point_x < target_downward_right_bound_x ) ||
            ( center_bottom_point_x > target_downward_left_bound_x && center_bottom_point_x < target_downward_right_bound_x )
        )
        {
            if
            (
                top_points_y < target_upward_bottom_bound_y &&
                top_points_y > target_upward_top_bound_y
            )
            {
                overlap_y_top = target_upward_bottom_bound_y - top_points_y;
            }
        }

        if
        (
            ( left_point_y_top > left_target_bound_y_top && left_point_y_top < left_target_bound_y_bottom ) ||
            ( left_point_y_bottom > left_target_bound_y_top && left_point_y_bottom < left_target_bound_y_bottom ) ||
            ( left_point_y_center > left_target_bound_y_top && left_point_y_center < left_target_bound_y_bottom )
        )
        {
            if
            (
                left_point_x > left_target_bound_x_left &&
                left_point_x < left_target_bound_x_right
            )
            {
                overlap_x_left = left_target_bound_x_right - left_point_x;
            }
        }

        if
        (
            ( left_point_y_top > left_target_bound_y_top && left_point_y_top < left_target_bound_y_bottom ) ||
            ( left_point_y_bottom > left_target_bound_y_top && left_point_y_bottom < left_target_bound_y_bottom ) ||
            ( left_point_y_center > left_target_bound_y_top && left_point_y_center < left_target_bound_y_bottom )
        )
        {
            if
            (
                right_point_x > right_target_bound_x_left &&
                right_point_x < right_target_bound_x_right
            )
            {
                overlap_x_right = right_point_x - right_target_bound_x_left;
            }
        }
        */



        return Collision( overlap_x_left, overlap_x_right, overlap_y_top, overlap_y_bottom );
    };

    int Object::xPrevSubPixels() const
    {
        return x_prev_;
    };

    int Object::yPrevSubPixels() const
    {
        return y_prev_;
    };

    int Object::xPrevPixels() const
    {
        return Unit::SubPixelsToPixels( x_prev_ );
    };

    int Object::yPrevPixels() const
    {
        return Unit::SubPixelsToPixels( y_prev_ );
    };

    int Object::rightPixels() const
    {
        return Unit::SubPixelsToPixels( hitboxRight() );
    };

    int Object::bottomPixels() const
    {
        return Unit::SubPixelsToPixels( hitboxBottom() );
    };

    int Object::rightSubPixels() const
    {
        return hitboxRight();
    };

    int Object::bottomSubPixels() const
    {
        return hitboxBottom();
    };

    int Object::rightPrevPixels() const
    {
        return Unit::SubPixelsToPixels( hitboxPrevRight() );
    };

    int Object::bottomPrevPixels() const
    {
        return Unit::SubPixelsToPixels( hitboxPrevBottom() );
    };

    int Object::rightPrevSubPixels() const
    {
        return hitboxPrevRight();
    };

    int Object::bottomPrevSubPixels() const
    {
        return hitboxPrevBottom();
    };

    SDL_Rect& Object::boundBox()
    {
        return hit_box_;
    };
