
// Name
//===================================
//
// Object
//

#ifndef OBJECT_H
#define OBJECT_H


// FORWARD DECLARATIONS
//===================================

    class Camera;
    class Collision;
    class Graphics;
    class Input;


// DEPENDENCIES
//===================================

    #include "direction.h"
    #include <SDL2/SDL.h>
    #include "unit.h"


// CLASS
//===================================

    class Object
    {
        public:
            Object
            (
                int x = 0,
                int y = 0,
                int width = 16,
                int height = 16
            );
            ~Object();

            SDL_Rect hit_box_;

            virtual void render( Graphics& graphics, Camera& camera, bool priority = false ) = 0;
            const Collision collide( Object* other ) const;

            SDL_Rect& boundBox();

            int xPixels() const;
            int yPixels() const;
            int xSubPixels() const;
            int ySubPixels() const;

            int xPrevSubPixels() const;
            int yPrevSubPixels() const;
            int xPrevPixels() const;
            int yPrevPixels() const;

            int heightPixels() const;
            int widthPixels() const;
            int heightSubPixels() const;
            int widthSubPixels() const;

            int hitboxHalfWidth() const;
            int hitboxHalfHeight() const;
            int hitboxCenterX() const;
            int hitboxCenterY() const;

            int hitboxLeft() const;
            int hitboxRight() const;
            int hitboxTop() const;
            int hitboxBottom() const;

            int hitboxPrevLeft() const;
            int hitboxPrevRight() const;
            int hitboxPrevTop() const;
            int hitboxPrevBottom() const;

            int rightPixels() const;
            int bottomPixels() const;
            int rightSubPixels() const;
            int bottomSubPixels() const;
            int rightPrevPixels() const;
            int bottomPrevPixels() const;
            int rightPrevSubPixels() const;
            int bottomPrevSubPixels() const;


        protected:
            int x_prev_ = -123456789;
            int y_prev_ = -123456789;
            const int original_height_;
            const int original_width_;
            const int original_x_;
            const int original_y_;
    };

#endif // OBJECT_H

