

// Name
//===================================
//
// Palette
//


// DEPENDENCIES
//===================================

    #include "game.h"
    #include "palette.h"
    #include <iostream>


// STATIC PROPERTIES
//===================================


// METHODS
//===================================

    Palette::Palette( PaletteType p, unsigned int bg )
    :
        colors_ ( PALETTE_SETS[ testPal( p ) ] ),
        bg_ ( bg )
    {};

    Palette::Palette( PaletteSet p )
    :
        colors_ ( PALETTE_SETS[ testPal( p.type ) ] ),
        bg_ ( p.bg )
    {};

    void Palette::changePalette( PaletteType p, unsigned int bg )
    {
        createPalette( p );
        bg_ = bg;
    };

    void Palette::changePalette( PaletteSet p )
    {
        createPalette( p.type );
        bg_ = p.bg;
    };

    Palette::~Palette() {};

    void Palette::applyPalette( SDL_Surface* s ) const
    {
        if ( s != nullptr )
        {
            for ( int i = 0; i < COLOR_LIMIT; ++i )
            {
                // Obviously we should abort if we reach beyond the # o' colors in the surface's array.
                if ( i > s->format->palette->ncolors )
                {
                    return;
                }
                else
                {
                    s->format->palette->colors[i] = colors_[i];
                }
            }
        }
    };

    SDL_Color& Palette::getColor( unsigned int n )
    {
        if ( n > 0 && n < COLOR_LIMIT )
            return colors_[ n ];
        else
            return colors_[ 1 ];
    };

    SDL_Color& Palette::bg()
    {
        if ( bg_ > 0 )
        {
            if ( bg_ >= COLOR_LIMIT )
            {
                if ( Game::nextFrame( 256, 8 ) )
                    return colors_[ 1 ];
                else
                    return colors_[ 3 ];
            }
            else
            {
                return colors_[ bg_ ];
            }
        }
        else
        {
            return colors_[ 0 ];
        }
    };

    unsigned int Palette::testColor( unsigned int n ) const
    {
        if ( n > 0 && n < COLOR_LIMIT )
            return n;
        else
            return 1;
    };


    void Palette::createPalette( PaletteType p )
    {
        for ( int i = 0; i < COLOR_LIMIT; ++i )
        {
            colors_[ i ] = PALETTE_SETS[ p ][ i ];
        }
    };

    Uint8 Palette::bgR() { return bg().r; };
    Uint8 Palette::bgG() { return bg().g; };
    Uint8 Palette::bgB() { return bg().b; };
    Uint8 Palette::bgA() { return bg().a; };

    Palette::PaletteSet Palette::defaultPaletteSet() { return { PaletteType::GRAYSCALE, 1 }; };

    Palette::PaletteType Palette::testPal( Palette::PaletteType p ) const
    {
        if ( p >= 0 && p < PALETTE_LIMIT )
        {
            return p;
        }
        else
        {
            return PaletteType::GRAYSCALE;
        }
    };
