

// Name
//===================================
//
// Palette
//


#ifndef PALETTE_H
#define PALETTE_H


// FORWARD DECLARATIONS
//===================================


// DEPENDENCIES
//===================================

    #include <SDL2/SDL_image.h>
    #include "timers/timer_repeat.h"


// CLASS
//===================================

    class Palette
    {
        public:
            static const Uint8 DEFAULT_VALUE = 255;

            // Don't apply specific #s.
            enum PaletteType
            {
                GRAYSCALE,
                CLASSIC_GREEN,
                MIDNIGHT_BLUE,
                DUSTY_BROWN,
                SUNNY_YELLOW,
                ICY_BLUE,
                MOUNTAIN_RED,
                MUSTARD_ORANGE,
                POISON_PURPLE,
                __NULL // Must be last.
            };

            struct PaletteSet
            {
                PaletteType type;
                int bg;
            };

            Palette( PaletteType p = PaletteType::GRAYSCALE, unsigned int bg = 1 );
            Palette( PaletteSet p );
            ~Palette();

            void changePalette( PaletteType p, unsigned int bg = 1 );
            void changePalette( PaletteSet p );
            void applyPalette( SDL_Surface* s ) const;
            SDL_Color& getColor( unsigned int n = 1 );
            SDL_Color& bg();
            Uint8 bgR();
            Uint8 bgG();
            Uint8 bgB();
            Uint8 bgA();
            static PaletteSet defaultPaletteSet();

        private:
            static const int COLOR_LIMIT = 7;
            static const int PALETTE_LIMIT = 9;
            const SDL_Color PALETTE_SETS[ PALETTE_LIMIT ][ COLOR_LIMIT ] =
            {
                {
                    { 255, 255, 255, 0   },
                    { 255, 255, 255, 255 },
                    { 161, 161, 161, 255 },
                    { 121, 121, 121, 255 },
                    {  81,  81,  81, 255 },
                    {  40,  40,  40, 255 },
                    {   0,   0,   0, 255 }
                },
                {
                    { 255, 255, 255, 0   },
                    { 255, 255, 255, 255 },
                    { 190, 241,  51, 255 },
                    { 139, 172,  15, 255 },
                    {  48,  98,  48, 255 },
                    {  15,  56,  15, 255 },
                    {   0,   0,   0, 255 }
                },
                {
                    { 255, 255, 255, 0   },
                    { 255, 255, 255, 255 },
                    { 234, 195, 255, 255 },
                    { 151, 120, 198, 255 },
                    { 105,  80, 130, 255 },
                    {  51,  36,  42, 255 },
                    {   0,   0,   0, 255 }
                },
                {
                    { 255, 255, 255, 0   },
                    { 255, 255, 255, 255 },
                    { 234, 230, 153, 255 },
                    { 183, 162, 102, 255 },
                    { 141, 105,  61, 255 },
                    {  94,  43,  14, 255 },
                    {   0,   0,   0, 255 }
                },
                {
                    { 255, 255, 255, 0   },
                    { 255, 255, 255, 255 },
                    { 227, 213, 136, 255 },
                    { 160, 146,  95, 255 },
                    { 105,  90,  61, 255 },
                    {  49,  34,  28, 255 },
                    {   0,   0,   0, 255 }
                },
                {
                    { 255, 255, 255, 0   },
                    { 255, 255, 255, 255 },
                    { 175, 188, 217, 255 },
                    { 133, 139, 159, 255 },
                    {  98,  94, 107, 255 },
                    {  51,  39,  43, 255 },
                    {   0,   0,   0, 255 }
                },
                {
                    { 255, 255, 255, 0   },
                    { 255, 255, 255, 255 },
                    { 255, 183, 185, 255 },
                    { 222, 123, 124, 255 },
                    { 171,  73,  72, 255 },
                    {  28,  26,  27, 255 },
                    {   0,   0,   0, 255 }
                },
                {
                    { 255, 255, 255, 0   },
                    { 255, 255, 255, 255 },
                    { 248, 192, 136, 255 },
                    { 248, 128,  79, 255 },
                    { 170,  79,  43, 255 },
                    {  47,  29,  25, 255 },
                    {   0,   0,   0, 255 }
                },
                {
                    { 255, 255, 255, 0   },
                    { 255, 255, 255, 255 },
                    { 203, 165, 236, 255 },
                    { 160,  71, 236, 255 },
                    {  81,  42,  94, 255 },
                    {  43,  25,  32, 255 },
                    {   0,   0,   0, 255 }
                }
            };

            SDL_Color colors_[ COLOR_LIMIT ];
            unsigned int bg_ = 1;

            void createPalette( PaletteType p );
            unsigned int testColor( unsigned int n ) const;
            Palette::PaletteType testPal( Palette::PaletteType p ) const;
    };

#endif // PALETTE_H


