



// Name
//===================================
//
// PlayerSprite
//


// DEPENDENCIES
//===================================

    #include "block.h"
    #include "camera.h"
    #include "collision.h"
    #include "game.h"
    #include "event_system.h"
    #include "input.h"
    #include "inventory.h"
    #include "level.h"
    #include "map.h"
    #include "pause_state.h"
    #include "player_sprite.h"
    #include "player_graphics.h"
    #include "sprite_graphics.h"


// STATIC PROPERTIES
//===================================


// METHODS
//===================================

    PlayerSprite::PlayerSprite( int x, int y )
    :
        Sprite( new PlayerGraphics(), x, y, 14, 23, SpriteType::HERO )
    {};

    PlayerSprite::~PlayerSprite() {};

    void PlayerSprite::customUpdate( Input& input, Camera& camera, Map& lvmap, Game& game, EventSystem& events )
    {
        if ( !events.won_ && !isDead() )
        {
            if ( input.pressed( Input::Action::MENU ) )
            {
                jump_lock_ = true;
                game.pushState( std::unique_ptr<GameState> ( new PauseState( lvmap.palette(), events ) ) );
            }
            if ( input.held( Input::Action::MOVE_DOWN ) )
            {
                duck();
            }
            else
            {
                unduck();
            }

            if ( !cant_stop_ )
            {
                if ( input.held( Input::Action::MOVE_LEFT ) && !input.held( Input::Action::MOVE_RIGHT ) && ( !isDucking() || !onGround() ) )
                {
                    moveLeft();
                }
            }

            if ( input.held( Input::Action::MOVE_RIGHT ) && !input.held( Input::Action::MOVE_LEFT ) && ( !isDucking() || !onGround() ) )
            {
                moveRight();
            }

            if ( !cant_stop_ && !is_sliding_ )
            {
                if ( !input.held( Input::Action::MOVE_RIGHT ) && !input.held( Input::Action::MOVE_LEFT ))
                {
                    if ( isDucking() )
                        stopDucking();
                    else
                        stop();
                }
            }

            if ( input.held( Input::Action::RUN ) )
            {
                run();
            }

            if ( input.held( Input::Action::EXTRA1 ) )
            {
                camera.moveLeft();
            }

            if ( input.held( Input::Action::EXTRA2 ) )
            {
                camera.moveRight();
            }

            if ( input.held( Input::Action::RUN ) )
            {
                run();
            }

            if ( input.held( Input::Action::JUMP ) )
            {
                    jump();
                    jump_lock_ = true;

                slowFall();
            }

            if ( input.released( Input::Action::JUMP ) )
            {
                jump_lock_ = false;
            }

            if ( input.held( Input::Action::MOVE_UP ) || input.held( Input::Action::MOVE_DOWN ) )
            {
                if ( touching_ladder_ )
                    grabLadder();
            }

            if ( input.held( Input::Action::MOVE_UP ) )
            {
                moveUp();

                if ( onGround() && !isMoving() )
                    camera.moveUp();
            }

            if ( input.released( Input::Action::MOVE_UP ) )
            {
                stopLookingUp();
            }

            if ( input.held( Input::Action::MOVE_DOWN ) )
            {
                moveDown();

                if ( onGround() && !isMoving() )
                {
                    camera.moveDown();
                }
            }

        // In case player gets stuck 'tween 2 blocks with just 1 empty block 'tween them vertically.
        // Makes player automatically move to the side till out o' space ala Super Mario Bros. 3.
        if ( collide_top_prev_ && collide_bottom_prev_ && !isDucking() )
        {
            if ( directionX() == Direction::Horizontal::RIGHT )
                moveRight();
            else
                moveLeft();
        }

        if ( !touching_ladder_ )
            releaseLadder();

        boundaries( camera );

        }

        if ( isDead() )
        {
            unduck();
            jump();
            interact_ = false;
        }

        if ( fellInBottomlessPit( lvmap ) )
            events.failed_ = true;

    };

    void PlayerSprite::customInteract( Collision& my_collision, Collision& their_collision, Sprite& them )
    {
    };
