
// Name
//===================================
//
// SpriteGraphics
//

#ifndef SPRITE_GRAPHICS_H
#define SPRITE_GRAPHICS_H


// FORWARD DECLARATIONS
//===================================

    class Camera;
    class Sprite;


// DEPENDENCIES
//===================================

    #include "graphics.h"
    #include "timers/timer_repeat.h"


// CLASS
//===================================

    class SpriteGraphics
    {
        public:
            SpriteGraphics
            (
                Graphics::SpriteSheet texture = Graphics::SpriteSheet::LIMIT,
                int current_frame_x = 0,
                int current_frame_y = 0,
                bool flip_x = false,
                bool flip_y = false,
                double rotation = 0,
                bool priority = false,
                int x_adjustment = 0,
                int y_adjustment = 0,
                int w_adjustment = 0,
                int h_adjustment = 0
            );

            bool flip_x_;
            bool flip_y_;
            double rotation_;
            bool priority_;
            int current_frame_x_;
            int current_frame_y_;
            int x_adjustment_;
            int y_adjustment_;
            int w_adjustment_;
            int h_adjustment_;

            virtual ~SpriteGraphics();
            virtual void update( Sprite& sprite );
            void render( Graphics& graphics, SDL_Rect bound_box, Camera* camera = nullptr, bool priority = false );
            SDL_Rect adjustBoundBox( SDL_Rect& r ) const;
            bool visible_ = true;

        protected:
            TimerRepeat animation_timer_;
            Graphics::SpriteSheet texture_ = Graphics::SpriteSheet::LIMIT;
            int prev_frame_x_ = 0;
            int prev_frame_y_ = 0;

            int cutoffLeft  ( int object_x, int camera_x ) const;
            int cutoffTop   ( int object_y, int camera_y ) const;
            int cutoffRight ( int object_x, int camera_x, int object_w, int camera_w ) const;
            int cutoffBottom( int object_y, int camera_y, int object_h, int camera_h ) const;
            int cutoffLeft  ( SDL_Rect& obj_r, Camera& c ) const;
            int cutoffTop   ( SDL_Rect& obj_r, Camera& c ) const;
            int cutoffRight ( SDL_Rect& obj_r, Camera& c ) const;
            int cutoffBottom( SDL_Rect& obj_r, Camera& c ) const;
    };


#endif // SPRITE_GRAPHICS_H

