



// Name
//===================================
//
// Text
//

#ifndef TEXT_H
#define TEXT_H


// FORWARD DECLARATIONS
//===================================

    class Camera;
    class Graphics;
    class TextComponent;


// DEPENDENCIES
//===================================

    #include "counter.h"
    #include <map>
    #include <SDL2/SDL.h>
    #include <string>
    #include "timers/timer_repeat.h"


// CLASS
//===================================

    class Text
    {
        public:
            enum CharSymbols
            {
                PTS = 0,
                HEART = 1,
                RIGHT_ARROW = 2,
                LEFT_ARROW = 3,
                DOWN_ARROW = 4,
                UP_ARROW = 5
            };

            enum FontShade
            {
                BLACK = 0,
                DARK_GRAY = 1,
                DARK_MID_GRAY = 2,
                LIGHT_MID_GRAY = 3,
                LIGHT_GRAY = 4,
                WHITE = 5,
                __NULL = 6
            };

            enum FontAlign
            {
                FA_LEFT,
                FA_CENTER,
                FA_RIGHT
            };

            Text
            (
                std::string words,
                int x = 0,
                int y = 0,
                FontShade shade = FontShade::BLACK,
                FontAlign align = FontAlign::FA_LEFT,
                bool blink = false,
                TextComponent* component = nullptr,
                int line_limit = -1
            );

            ~Text();
            std::string words_;
            int x_ = 0;
            int y_ = 0;
            void update();
            void render( Graphics& graphics, Camera* camera = nullptr, FontShade shade = FontShade::__NULL );
            static void renderNumber( Graphics& graphics, int n, int x, int y, int digits = -1, FontShade shade = FontShade::BLACK, Camera* camera = nullptr );
            void moveLeft( int n = 1 );
            void moveRight( int n = 1 );
            int right() const;
            int x() const;
            static void renderText( Graphics& graphics, std::string text, int x, int y, Camera* camera = nullptr, FontShade shade = FontShade::BLACK, int line_limit = -1 );
            static std::string stringifyNum( int n );
            static std::string formatNumCommas( int n );
            static std::string formatNumDigitPadding( int n, int digits );
            static std::string formatNumCommas( std::string text );
            static std::string formatNumDigitPadding( std::string text, int digits );
            static std::string timeToString( std::string seconds, std::string minutes, int minutes_padding = 1 );
            static std::string timeToString( int seconds, int minutes, int minutes_padding = 1 );
            static int getDigit( int n, int d, int remain = 10 );
            static int numODigits( int n );
            bool flipped() const;

        private:
            static const int CHAR_SIZE_PIXELS = 8;
            static const int NUM_OF_FONT_SHADES = 6;
            static const int CHARSET_WIDTH_MINI_BLOCKS = 32;
            static std::map<char, int> char_conversion_;
            FontShade shade_;
            FontAlign align_;
            bool blink_;
            Counter blink_counter_ = Counter( 0, NUM_OF_FONT_SHADES-1, 0, true );
            TimerRepeat blink_timer_;
            Direction::Vertical blink_dir_ = Direction::Vertical::UP;
            TextComponent* component_;
            bool flipped_ = false;
            int line_limit_;

            static int frameX( int n );
            static int frameY( int n, FontShade shade );
            static int shadeOffset( FontShade shade );
            int shadeNum() const;
            int centerX() const;
    };

#endif // TEXT_H

